"""Türkçe dil paketi."""

LANG_TR = {
    "code": "tr",
    "name": "Turkish",
    "native_name": "Türkçe",

    # Common trigrams for language detection
    "trigrams": [
        "lar", "ler", "bir", "de ", "da ", "ve ", "in ", "an ",
        "en ", "ir ", "ar ", "ini", "ını", "ile", "ası", "esi",
        "lar", "ler", "dır", "dir", "tir", "tır", "yor", "mak",
        "mek", "ama", "eme", "ine", "ını", "ığı", "isi", "üzü",
        "ılı", "ili", "olu", "ülü", "aya", "eye",
    ],

    # Stop words
    "stop_words": {
        "bir", "bu", "şu", "o", "ve", "ile", "için", "de", "da",
        "den", "dan", "ne", "ya", "hem", "ama", "fakat", "ancak",
        "çünkü", "eğer", "gibi", "kadar", "daha", "en", "çok",
        "az", "her", "hiç", "bazı", "tüm", "bütün", "olan",
        "olarak", "üzere", "göre", "karşı", "sonra",
        "önce", "arasında", "üzerinde", "altında", "yanında",
        "içinde", "dışında", "hakkında", "tarafından", "dolayı",
        "rağmen", "beri", "boyunca", "ise", "ki", "mi", "mı",
        "mu", "mü", "bile", "sadece", "yalnız",
    },

    # Bureaucratic / overly formal words -> simpler alternatives
    "bureaucratic": {
        "kullanmak": ["kullanmak", "yararlanmak"],
        "istifade etmek": ["yararlanmak", "faydalanmak"],
        "icra etmek": ["yapmak", "uygulamak"],
        "tatbik etmek": ["uygulamak", "yapmak"],
        "teşvik etmek": ["özendirmek", "desteklemek"],
        "başlatmak": ["başlamak", "açmak"],
        "nihayetlendirmek": ["bitirmek", "sonlandırmak"],
        "ikmal etmek": ["tamamlamak", "bitirmek"],
        "tahakkuk ettirmek": ["gerçekleştirmek", "başarmak"],
        "teyit etmek": ["doğrulamak", "onaylamak"],
        "tetkik etmek": ["incelemek", "araştırmak"],
        "temin etmek": ["sağlamak", "bulmak"],
        "iktisap etmek": ["elde etmek", "almak"],
        "tahakkuk etmek": ["gerçekleşmek", "olmak"],
        "inşa etmek": ["kurmak", "yapmak"],
        "tanzim etmek": ["düzenlemek", "hazırlamak"],
        "terakkî etmek": ["ilerlemek", "gelişmek"],
        "idame ettirmek": ["sürdürmek", "devam ettirmek"],
        "ıslah etmek": ["düzeltmek", "iyileştirmek"],
        "takviye etmek": ["güçlendirmek", "pekiştirmek"],
        "kavramak": ["anlamak", "algılamak"],
        "idrak etmek": ["anlamak", "kavramak"],
        "hükmetmek": ["karar vermek", "yargılamak"],
        "mukabele etmek": ["karşılık vermek", "yanıtlamak"],
        "talep etmek": ["istemek", "rica etmek"],
        "müracaat etmek": ["başvurmak", "dilekçe vermek"],
        "tasdik etmek": ["onaylamak", "kabul etmek"],
        "ruhsat vermek": ["izin vermek", "onaylamak"],
        "tebliğ etmek": ["bildirmek", "haber vermek"],
        "rapor etmek": ["bildirmek", "aktarmak"],
        "irtibat kurmak": ["iletişime geçmek", "bağlantı kurmak"],
        "iştirak etmek": ["katılmak", "dahil olmak"],
        "teşrik-i mesai etmek": ["iş birliği yapmak", "birlikte çalışmak"],
        "hizmet etmek": ["katkıda bulunmak", "yardımcı olmak"],
        "vuku bulmak": ["olmak", "meydana gelmek"],
        "mevcut olmak": ["var olmak", "bulunmak"],
        "tenkis etmek": ["azaltmak", "kısmak"],
        "artırmak": ["çoğaltmak", "yükseltmek"],
        "tevsi etmek": ["genişletmek", "büyütmek"],
        "daraltmak": ["küçültmek", "azaltmak"],
        "birleştirmek": ["bir araya getirmek", "kaynaştırmak"],
        "tahlil etmek": ["çözümlemek", "incelemek"],
        "takdir etmek": ["değer biçmek", "beğenmek"],
        "tadil etmek": ["değiştirmek", "düzeltmek"],
        "tebdil etmek": ["değiştirmek", "dönüştürmek"],
        "tayin etmek": ["belirlemek", "atamak"],
        "ithal etmek": ["getirmek", "almak"],
        "ilga etmek": ["kaldırmak", "iptal etmek"],
        "tevzi etmek": ["dağıtmak", "paylaştırmak"],
        "idare etmek": ["yönetmek", "çekip çevirmek"],
        "tedvir etmek": ["yürütmek", "işletmek"],
        "murakabe etmek": ["denetlemek", "kontrol etmek"],
        "tensip etmek": ["uygun görmek", "seçmek"],
        "arz etmek": ["sunmak", "göstermek"],
        "muamele etmek": ["işlem yapmak", "davranmak"],
        "muhafaza etmek": ["korumak", "saklamak"],
        "iade etmek": ["geri vermek", "geri göndermek"],
        "intikal etmek": ["geçmek", "aktarılmak"],
        "istimal etmek": ["kullanmak", "değerlendirmek"],
        "farz etmek": ["varsaymak", "kabul etmek"],
        "hususiyle": ["özellikle", "bilhassa"],
        "binaenaleyh": ["bu nedenle", "dolayısıyla"],
        "maamafih": ["bununla birlikte", "yine de"],
        "bilahare": ["daha sonra", "ardından"],
        "muahede": ["anlaşma", "sözleşme"],
        "mütalaa etmek": ["düşünmek", "değerlendirmek"],
        "ihtiva etmek": ["içermek", "kapsamak"],
        "tefrik etmek": ["ayırmak", "ayrım yapmak"],
        "tahsis etmek": ["ayırmak", "vermek"],
        "teftiş etmek": ["denetlemek", "kontrol etmek"],
        "temadi etmek": ["sürmek", "devam etmek"],
        "tebaruz etmek": ["öne çıkmak", "belirmek"],
        "tekerrur etmek": ["tekrarlanmak", "yinelenmek"],
        "teferruat": ["ayrıntı", "detay"],
        "mukâyese etmek": ["karşılaştırmak", "kıyaslamak"],
        "müsaade etmek": ["izin vermek", "bırakmak"],
        "muvaffak olmak": ["başarmak", "becerebilmek"],
        "müracaat": ["başvuru", "dilekçe"],
        "tashih etmek": ["düzeltmek", "onarmak"],
        "teşebüs etmek": ["girişmek", "başlamak"],
        "istihsal etmek": ["üretmek", "elde etmek"],
        "tenvir etmek": ["aydınlatmak", "bilgilendirmek"],
        "istişare etmek": ["danışmak", "fikir almak"],
        "tedavi etmek": ["iyileştirmek", "sağaltmak"],
        "istihdam etmek": ["çalıştırmak", "görevlendirmek"],
        "ibraz etmek": ["göstermek", "sunmak"],
        "ihdas etmek": ["oluşturmak", "kurmak"],
        "istinas etmek": ["dayanmak", "güvenmek"],
        "terfi ettirmek": ["yükseltmek", "terfi vermek"],
        "tekamul": ["gelişme", "ilerleme"],
        "münhasıran": ["yalnızca", "sadece"],
        "mahiyet": ["nitelik", "içyüz"],
        "ahval": ["durumlar", "koşullar"],
        "mütemadiyen": ["sürekli", "durmadan"],
        "müteakiben": ["ardından", "hemen sonra"],
        "mütecaviz": ["saldırgan", "agresif"],
        "salahiyet": ["yetki", "hak"],
        "havi": ["içeren", "kapsayan"],
        "celp etmek": ["çekmek", "getirmek"],
        "der meyan etmek": ["belirtmek", "söylemek"],
        "tahtına": ["yerine", "karşılığında"],
        "meşru": ["yasaya uygun", "haklı"],
        "intibak etmek": ["uyum sağlamak", "alışmak"],
        "icraat": ["çalışma", "faaliyet"],
        "tedbir almak": ["önlem almak", "hazırlık yapmak"],
        "müzakere etmek": ["görüşmek", "müzâkere yapmak"],
        "meblag": ["tutar", "miktar"],
        "mukâvele": ["sözleşme", "anlaşma"],
        "mütevelli": ["sorumlu", "yönetici"],
        "hükümsüz kılmak": ["geçersiz saymak", "iptal etmek"],
        "beyan etmek": ["açıklamak", "bildirmek"],
        "istikamet": ["yön", "doğrultu"],
        "tazyik etmek": ["baskı yapmak", "zorlama yapmak"],
        "tevâzü": ["alçakgönüllülük", "mütevazılık"],
        "feragat etmek": ["vazgeçmek", "bırakmak"],
        "mütehâkim olmak": ["hâkim olmak", "egemen olmak"],
        "müesses": ["kurulu", "görevli"],
        "sarf etmek": ["harcamak", "kullanmak"],
        "mütearife": ["açık gerçek", "bedihiyet"],
        "ilân etmek": ["duyurmak", "açıklamak"],
        "infaz etmek": ["uygulamak", "yerine getirmek"],
        "tahkik etmek": ["soruşturmak", "araştırmak"],
    },

    # Bureaucratic phrases -> simpler versions
    "bureaucratic_phrases": {
        "onayınıza sunulmuştur": ["onayınıza sunuyoruz", "onayınızı bekliyoruz"],
        "gereğini arz ederim": ["gereğini rica ederim", "yapılmasını istiyorum"],
        "bilgilerinize sunulur": ["bilginize", "size bildiriyoruz"],
        "yukarıda belirtilen hususlar": ["yukarıda söylenen konular", "anlattığımız konular"],
        "ilgi yazınız gereğince": ["yazdığınız gibi", "yazınıza göre"],
        "söz konusu husus": ["bu konu", "bahsedilen mesele"],
        "mezkûr tarihte": ["belirtilen tarihte", "o tarihte"],
        "muktezası gereğince": ["gereği doğrultusunda", "gerektiği şekilde"],
        "zikredilen hususlar": ["adı geçen konular", "söylenen şeyler"],
        "meri mevzuat çerçevesinde": ["yürürlükteki yasalara göre", "geçerli kurallara göre"],
        "sair hükümler saklı kalmak kaydıyla": [
            "diğer kurallar geçerli olmak üzere",
            "geri kalanı saklı kalmak şartıyla",
        ],
        "mezkûr kanun hükmünde": ["söz konusu yasa uyarınca", "o yasaya göre"],
        "yukarıda arz edilen": ["yukarıda anlatılan", "yukarıda belirtilen"],
        "bilgi edinilmesi hususunda": ["bilmeniz için", "haberiniz olsun diye"],
        "gereği bilgilerinize": ["bilginize", "bilmeniz için"],
        "emrilerinize arz olunur": ["bilginize sunuyoruz", "arz ediyoruz"],
        "yapılmasını teminen": ["yapılması için", "sağlamak amacıyla"],
        "bahse konu olan": ["söz konusu", "sözü edilen"],
        "tarafınızca uygun görüldüğü takdirde": [
            "siz uygun görürseniz",
            "onaylarsanız",
        ],
        "iş bu yazımız ile": ["bu yazıyla", "bu mektupla"],
        "netice itibarıyla": ["sonuç olarak", "sonunda"],
        "hususun bilgilerinize sunulmasını": [
            "konuyu bilmenizi",
            "durumdan haberdar olmanızı",
        ],
        "ivedilikle gereğinin yapılması": [
            "hemen yapılması",
            "acilen halledilmesi",
        ],
        "mevzubahis olan": ["konu olan", "söz edilen"],
        "vaki olan başvuru": ["yapılan başvuru", "gelen başvuru"],
        "tedbirler alınması hususunda": ["önlem alınması için", "tedbir almak üzere"],
        "lüzumu hâlinde": ["gerekirse", "gerektiğinde"],
        "mutabık kalınması hâlinde": ["anlaşılırsa", "uzlaşma sağlanırsa"],
        "aşağıda belirtilen hususlar": ["aşağıda yazılanlar", "aşağıdaki konular"],
        "yukarıda izah edilen": ["yukarıda açıklanan", "anlatıldığı gibi"],
        "hususun takdirlerine arz olunmasını": [
            "değerlendirmenize sunuyoruz",
            "bilginize sunuyoruz",
        ],
        "yazımıza cevaben": ["yazınıza yanıt olarak", "mektubunuza karşılık"],
        "aşağıdaki şekilde işlem yapılması": ["aşağıdaki gibi yapılması", "şu şekilde uygulanması"],
        "muhatap kurum tarafından": ["ilgili kurum tarafından", "karşı taraftan"],
        "gereğinin ifası hususunda": ["gereğinin yapılması için", "gereken adımların atılması"],
        "müddetinde eda edilmesi": ["süresinde tamamlanması", "zamanında yapılması"],
        "tedabir ittihaz olunması": ["önlem alınması", "tedbir alınması"],
        "maslahatgüzarlar aracılığıyla": ["diplomatik yollarla", "aracılar yoluyla"],
        "itina gösterilmesi rica olunur": [
            "özen gösterilmesini rica ediyoruz",
            "dikkatli davranılmasını istiyoruz",
        ],
        "müştereken hareket edilmesi": ["birlikte hareket edilmesi", "ortak çalışılması"],
        "tahsil edilmesi gereken": ["alınması gereken", "toplanması gereken"],
        "ilgililerin bilgisine sunulur": ["ilgililere bildirilir", "ilgili kişilere duyurulur"],
        "hak ve mükellefiyetleri": ["hak ve yükümlülükleri", "haklar ve ödevler"],
        "yükümlülüklerin ifasını": [
            "ödevlerin yerine getirilmesini",
            "sorumlulukların yerine getirilmesini",
        ],
        "tarafsızlığın muktezasınca": ["tarafsızlık gereği", "tarafsız şekilde"],
        "mücbir sebep hali dışında": ["zorunlu durumlar hariç", "olağanüstü durumlar dışında"],
        "teşkilâtın idamesi bakımından": ["kurumun devamı açısından", "yapının sürmesi için"],
    },

    # AI connector phrases -> replacements
    "ai_connectors": {
        "bununla birlikte": ["ama", "yine de", "ne var ki"],
        "buna rağmen": ["ama", "yine de", "gene de"],
        "bunun yanı sıra": ["ayrıca", "üstelik", "buna ek olarak"],
        "dahası": ["üstelik", "hem de", "bir de"],
        "buna ek olarak": ["ayrıca", "üstelik", "bir de"],
        "dolayısıyla": ["bu yüzden", "o nedenle", "bu sebeple"],
        "bu sebepten ötürü": ["bu yüzden", "o nedenle", "bundan dolayı"],
        "yani": ["kısacası", "başka bir deyişle", "özetle"],
        "başka bir deyişle": ["yani", "diğer bir ifadeyle", "kısacası"],
        "şüphesiz": ["elbette", "kesinlikle", "kuşkusuz"],
        "kuşkusuz": ["elbette", "şüphesiz", "hiç şüphe yok"],
        "sonuç olarak": ["özetle", "kısacası", "son söz olarak"],
        "nihayetinde": ["sonunda", "en sonunda", "neticede"],
        "her şeyden önce": ["öncelikle", "ilk olarak", "en başta"],
        "öte yandan": ["diğer taraftan", "buna karşılık", "ama"],
        "bunun aksine": ["tam tersine", "aksine", "buna karşın"],
        "genel olarak": ["genelde", "çoğunlukla", "genellikle"],
        "özellikle": ["bilhassa", "başta", "en çok"],
        "dikkat çekici olan": ["göze çarpan", "önemli olan", "ilginç olan"],
        "önemli olan şu ki": ["asıl mesele şu", "kilit nokta şu", "esas olan"],
        "özetle": ["kısaca", "toparlarsak", "kısa ve öz"],
        "bunun sonucunda": ["sonuç olarak", "bu yüzden", "neticesinde"],
        "özünde": ["temelde", "aslında", "esasen"],
        "kayda değer biçimde": ["önemli ölçüde", "belirgin şekilde", "fark edilir biçimde"],
        "buna göre": ["buna uygun olarak", "buna bağlı olarak", "ona göre"],
        "doğal olarak": ["elbette", "tabii ki", "hâliyle"],
        "alternatif olarak": ["başka bir seçenek olarak", "ya da", "yoksa"],
        "görünüşe göre": ["anlaşılan", "belli ki", "galiba"],
        "daha önce belirtildiği gibi": [
            "daha önce dediğimiz gibi",
            "önceki söylediğimiz üzere",
            "yukarıda da geçtiği gibi",
        ],
        "genel itibarıyla": ["bütünüyle", "toptan bakınca", "kabaca"],
        "toparlayacak olursak": ["özetlersek", "sonuç olarak", "genel değerlendirme yaparsak"],
        "kesinlikle": ["şüphesiz", "mutlaka", "tartışmasız"],
        "açıkça": ["belli ki", "görünür biçimde", "net biçimde"],
        "aynı zamanda": ["bir yandan da", "hem de", "üstelik"],
        "tersine": ["aksine", "tam tersi", "bilakis"],
        "benzer şekilde": ["aynı biçimde", "tıpkı öyle", "buna benzer olarak"],
        "temelden": ["kökten", "baştan", "esastan"],
        "hiç kuşkusuz": ["kesinlikle", "şüphesiz", "mutlaka"],
        "bu bağlamda": ["burada", "bu durumda", "bu açıdan"],
        "bu konuda": ["bu hususta", "bu mesele hakkında", "bununla ilgili"],
        "bunu dikkate alarak": ["bunu göz önünde bulundurarak", "buna istinaden", "buna dayanarak"],
        "aslında": ["gerçekte", "işin aslı", "doğrusu"],
        "ilginç olan şu ki": ["enteresan olan", "dikkat çekici olan", "şaşırtıcı olan"],
        "aynı şekilde": ["benzer biçimde", "tıpkı böyle", "aynen öyle"],
        "bu arada": ["bu esnada", "o sırada", "bu süreçte"],
        "daha da önemlisi": ["asıl önemli olan", "en önemlisi", "hele hele"],
        "somut olarak": ["net olarak", "açık biçimde", "spesifik olarak"],
        "doğaldır ki": ["elbette", "tabii", "hâliyle"],
        "ancak şunu belirtmek gerekir ki": [
            "ama şunu söylemek lazım",
            "fakat şunu da eklemeliyim",
            "bir şey daha var",
        ],
        "özellikle vurgulanmalıdır ki": [
            "bilhassa belirtmeliyim ki",
            "altını çizerek söylemeliyim",
            "önemle diyeyim ki",
        ],
        "tüm bunların ışığında": [
            "bütün bunlardan yola çıkarak",
            "bunlar göz önünde tutulursa",
            "bunlara bakarak",
        ],
        "şrafa koymak yerine": [
            "bir kenara bırakmak yerine",
            "yok saymak yerine",
            "görmezden gelmek yerine",
        ],
        "işin özüne bakarsak": ["esasa bakarsak", "asıl meseleye gelirsek", "konuya girirsek"],
        "peki ama şu var": [
            "ama bir şey var",
            "ne var ki",
            "fakat dikkat çeken bir şey",
        ],
        "ısrarla ifade edersek": [
            "bir kez daha söyleyelim",
            "tekrarlamak gerekirse",
            "yineleyerek söyleyelim",
        ],
        "ilk bakışta": ["görünüşte", "dışarıdan bakınca", "yüzeysel olarak"],
        "gelin görün ki": ["ama ne yazık ki", "ancak", "ne var ki"],
        "ne ilginçtir ki": ["enteresan olan", "şaşırtıcı biçimde", "dikkat çekici olan"],
        "hakkını teslim edelim": ["dürüst olalım", "adil olmak gerekirse", "kabul etmek lazım"],
        "en kötü senaryoda": ["en kötü ihtimalle", "en kötüsünde", "en olumsuz durumda"],
        "bir başka deyişle": ["yani", "kısaca", "başka bir anlatımla"],
        "tartışılması gereken": [
            "konuşulması gereken",
            "ele alınması gereken",
            "üzerinde durulması gereken",
        ],
        "bunun dışında": ["bundan başka", "bir de", "ayrıca"],
        "değerlendirildiğinde": ["ele alındığında", "incelendiğinde", "bakıldığında"],
        "nitekim": ["zaten", "nitekim öyle", "beklenildiği gibi"],
        "tümüyle": ["tamamen", "bütünüyle", "baştan aşağı"],
        "ne hazindir ki": ["ne yazık ki", "üzülerek söyleyeyim", "maalesef"],
    },

    # Synonyms for repetition reduction
    "synonyms": {
        "önemli": ["mühim", "kilit", "kritik", "hayati"],
        "büyük": ["kocaman", "devasa", "iri", "geniş"],
        "küçük": ["ufak", "minik", "cüce", "dar"],
        "iyi": ["güzel", "hoş", "harika", "mükemmel"],
        "kötü": ["berbat", "fena", "boktan", "rezil"],
        "hızlı": ["çabuk", "süratli", "tez", "acil"],
        "yavaş": ["ağır", "tembel", "sakin", "aheste"],
        "yeni": ["taze", "çağdaş", "modern", "son model"],
        "eski": ["kadim", "antik", "köhne", "geçmiş"],
        "güzel": ["hoş", "şık", "zarif", "alımlı"],
        "çirkin": ["biçimsiz", "betebeniz", "yakışıksız", "göze batmayan"],
        "güçlü": ["sağlam", "kuvvetli", "dayanıklı", "kudretli"],
        "zayıf": ["cılız", "güçsüz", "dermansız", "kırılgan"],
        "zor": ["güç", "çetin", "sarp", "ağır"],
        "kolay": ["basit", "zahmetsiz", "rahat", "yalın"],
        "başlamak": ["girişmek", "açmak", "koyulmak", "el atmak"],
        "bitmek": ["sona ermek", "tamamlanmak", "kapanmak", "noktalanmak"],
        "çalışmak": ["emek vermek", "uğraşmak", "didinmek", "mesai yapmak"],
        "söylemek": ["demek", "ifade etmek", "belirtmek", "dile getirmek"],
        "düşünmek": ["kafa yormak", "değerlendirmek", "tartmak", "kafasını kurcalamak"],
        "görmek": ["izlemek", "gözlemlemek", "bakmak", "fark etmek"],
        "almak": ["edinmek", "kapmak", "tutmak", "ele geçirmek"],
        "vermek": ["sunmak", "uzatmak", "bırakmak", "teslim etmek"],
        "gitmek": ["yola çıkmak", "yönelmek", "ayrılmak", "hareket etmek"],
        "gelmek": ["varmak", "ulaşmak", "çıkagelmek", "görünmek"],
        "bilmek": ["anlamak", "kavramak", "farkında olmak", "haberdar olmak"],
        "sevmek": ["hoşlanmak", "bayılmak", "tutulmak", "hayran olmak"],
        "istemek": ["arzu etmek", "dilemek", "talep etmek", "arzulamak"],
        "yapabilmek": ["becermek", "başarabilmek", "üstesinden gelmek", "hakkından gelmek"],
        "yer": ["mekân", "konum", "alan", "nokta"],
        "zaman": ["vakit", "süre", "dönem", "an"],
        "kişi": ["insan", "birey", "şahıs", "fert"],
        "iş": ["görev", "meslek", "uğraş", "faaliyet"],
        "sorun": ["mesele", "sıkıntı", "dert", "problem"],
        "çözüm": ["çare", "hal", "yol", "reçete"],
        "sonuç": ["netice", "çıktı", "ürün", "etki"],
        "neden": ["sebep", "gerekçe", "kök", "kaynak"],
        "yöntem": ["yol", "usul", "teknik", "strateji"],
        "fikir": ["düşünce", "görüş", "kanı", "tasarım"],
        "hedef": ["amaç", "gaye", "ulaşılmak istenen", "niyet"],
        "geliştirme": ["iyileştirme", "ilerleme", "yapım", "tasarım"],
        "araştırma": ["inceleme", "soruşturma", "analiz", "çalışma"],
        "rapor": ["bildiri", "sunum", "bülten", "özet"],
        "toplantı": ["görüşme", "buluşma", "oturum", "miting"],
        "plan": ["program", "taslak", "tasarı", "proje"],
        "karar": ["hüküm", "tercih", "yargı", "seçim"],
        "başarı": ["zafer", "kazanım", "muvaffakiyet", "erişim"],
        "başarısızlık": ["hezimet", "fiyasko", "hüsran", "çuvallamak"],
        "değişim": ["dönüşüm", "evrilme", "geçiş", "farklılaşma"],
        "artış": ["yükseliş", "tırmanış", "çoğalma", "büyüme"],
        "azalış": ["düşüş", "gerileme", "daralma", "erime"],
        "fırsat": ["şans", "imkân", "olanak", "vesile"],
        "mücadele": ["savaşım", "uğraş", "çaba", "kavga"],
        "bilgi": ["veri", "malumat", "enformasyon", "haber"],
        "konu": ["mesele", "gündem", "husus", "mevzu"],
        "durum": ["hâl", "koşul", "vaziyet", "tablo"],
        "ülke": ["memleket", "yurt", "vatan", "devlet"],
        "hükümet": ["yönetim", "idare", "otorite", "iktidar"],
        "toplum": ["halk", "cemiyet", "camia", "kamu"],
        "kültür": ["medeniyet", "gelenek", "değerler", "miras"],
        "bilim": ["fen", "ilim", "akademi", "araştırma"],
        "eğitim": ["öğretim", "tahsil", "terbiye", "yetişme"],
        "sağlık": ["afiyet", "esenlik", "zindelik", "sıhhat"],
        "yemek": ["yiyecek", "aş", "gıda", "sofra"],
        "su": ["akarsu", "sıvı", "içecek"],
        "ev": ["konut", "yuva", "mesken", "bark"],
        "yol": ["cadde", "güzergâh", "patika", "rota"],
        "kitap": ["eser", "yapıt", "cilt", "yayın"],
        "konuşma": ["söylev", "sohbet", "muhabbet", "laf"],
        "savaş": ["muharebe", "çatışma", "harb", "dövüş"],
        "barış": ["sulh", "huzur", "sükûnet", "selamet"],
        "özgürlük": ["hürriyet", "bağımsızlık", "serbestlik", "kurtuluş"],
        "adalet": ["hak", "doğruluk", "dürüstlük", "hakkaniyet"],
        "eğilim": ["trend", "yöneliş", "meyil", "eğret"],
        "bakış açısı": ["pencere", "gözlük", "perspektif", "açı"],
        "değer": ["kıymet", "anlam", "fayda", "bedel"],
        "yapı": ["iskelet", "çatı", "kurgu", "mimarlik"],
        "çevre": ["ortam", "etraf", "atmosfer", "ambiyans"],
        "tasarım": ["dizayn", "biçim", "görüntü", "estetik"],
        "veri": ["bilgi", "sayılar", "rakam", "enformasyon"],
        "kullanıcı": ["müşteri", "kişi", "faydalanan", "yürüüyen"],
        "performans": ["başarım", "verim", "gösterim", "aktivite"],
        "tepki": ["geri dönüş", "reaksiyon", "karşılık", "cevap"],
        "tartışma": ["tartışı", "diyalog", "müneşa", "ateşli konuşma"],
        "denge": ["uyum", "istikrar", "parallellik", "eşitlenme"],
        "risk": ["tehlike", "tehdit", "dezavantaj", "saklı tehlike"],
        "maliyet": ["masraf", "harcama", "gider", "bedel"],
        "ekonomi": ["ticaret", "mali tablo", "para düzen", "hazine"],
        "ağ": ["bağlantı", "şebeke", "network", "ilişki ağı"],
        "öngörü": ["tahmin", "kestirim", "vizyon", "önsezi"],
        "proje": ["girişim", "atılım", "iş", "çalışma"],
        "içerik": ["malzeme", "metin", "muhteva", "dolgu"],
        "süreç": ["hakikat", "tren", "akış", "dönüşüm"],
        "yaklaşım": ["tutum", "tavır", "yönelim", "bakış"],
        "esneklik": ["cilkallık", "uyum kabiliyeti", "kıvraklık", "oynak-lık"],
        "şeffaflık": ["açıklık", "görünürlük", "berraklık", "dürüstlük"],
        "motivasyon": ["heves", "istek", "güdü", "azim"],
        "kapasite": ["hacim", "güç", "yetkinlik", "oylum"],
        "potansiyel": ["güç", "olabilirlik", "yetenek", "gelecek vaadi"],
        "empati": ["duygudaşlık", "anlama", "hissetme", "hassasiyet"],
        "algı": ["farkındalık", "kavrama", "sezgi", "bilinç"],
        "geri bildirim": ["yorum", "değerlendirme", "görüş", "eleştiri"],
        "destek": ["yardım", "dayanışma", "arka", "kalkan"],
        "iletişim": ["haberleşme", "diyalog", "temas", "bağlantı"],
        "temel": ["esas", "kök", "ana", "dayanak"],
        "konsept": ["düşünce", "tasarı", "ana fikir", "tema"],
        "istikrar": ["süreklilik", "kararlılık", "durağanlık", "sağlamlık"],
        "yetenek": ["kabiliyet", "beceri", "hüner", "maharet"],
        "inceleme": ["irdeleme", "tetkik", "gözlem", "analiz"],
        "şirket": ["firma", "işletme", "kuruluş", "ticarethane"],
        "bütçe": ["kezik", "keziye", "maliyet planı", "para planı"],
        "strateji": ["yöntem", "taktik", "plan", "yaklaşım"],
        "hassasiyet": ["duyarlılık", "incelik", "titizlik", "narin"],
        "tecrübe": ["yaşantı", "birikim", "deneyim", "görgü"],
        "muhteva": ["iç", "öz", "ne var ne yok", "dolu"],
        "safhalar": ["aşamalar", "basamaklar", "adımlar", "kademeler"],
        "derece": ["seviye", "kademe", "düzey", "aşama"],
        "tür": ["çeşit", "cins", "tip", "kategori"],
        "itibar": ["saygınlık", "güvenilirlik", "nam", "prestij"],
    },

    # Sentence starters -> alternatives
    "sentence_starters": {
        "Bu": ["Şu", "O", "Söz konusu"],
        "Bu durum": ["Şu durum", "O durum", "Mevcut durum"],
        "O": ["Kendisi", "Bahsi geçen", "Söz konusu kişi"],
        "Onlar": ["Kendileri", "Bu kişiler", "Adı geçenler"],
        "Biz": ["Ekibimiz", "Hepimiz", "Grubumuz"],
        "Ben": ["Şahsen", "Bence", "Kendim"],
        "Ve": ["Ayrıca", "Üstelik", "Bunun yanında"],
        "Ama": ["Ancak", "Ne var ki", "Yine de"],
        "Ayrıca": ["Bunun yanı sıra", "Üstelik", "Dahası"],
        "Eğer": ["Şayet", "Diyelim ki", "Varsayalım ki"],
        "Çünkü": ["Zira", "Nedeni şu ki", "Şundan dolayı"],
        "Yani": ["Kısacası", "Bir başka deyişle", "Özetle"],
        "Örneğin": ["Mesela", "Söz gelimi", "Diyelim"],
        "Aslında": ["Gerçekte", "İşin doğrusu", "Hakikatte"],
        "Elbette": ["Tabii ki", "Doğal olarak", "Hiç kuşkusuz"],
        "Son zamanlarda": ["Son günlerde", "Yakın dönemde", "Bu aralar"],
        "Şimdi": ["Şu anda", "Hâlihazırda", "Bugün"],
        "Öncelikle": ["İlk olarak", "Her şeyden önce", "Evvela"],
        "Sonra": ["Ardından", "Akabinde", "Müteakiben"],
        "Son olarak": ["Nihayetinde", "En sonunda", "Bitirirken"],
        "Genellikle": ["Çoğunlukla", "Ekseriyetle", "Tipik olarak"],
        "Özellikle": ["Bilhassa", "Hele hele", "Başta"],
        "Sonuçta": ["Neticede", "Eninde sonunda", "Nihayetinde"],
        "Gerçekten": ["Hakikaten", "Sahiden", "Cidden"],
        "Temelde": ["Esasen", "Özünde", "Aslına bakarsanız"],
        "Neyse ki": ["Şükür ki", "İyi ki", "Bereket versin"],
        "Maalesef": ["Ne yazık ki", "Üzülerek", "Yazık ki"],
        "İlginç bir şekilde": ["Enteresan olarak", "Şaşırtıcı biçimde", "Dikkat çekici olarak"],
        "Kesinlikle": ["Mutlaka", "Tartışmasız", "Şüphesiz"],
        "Dikkat çekici olan": ["Göze çarpan", "İlginç olan", "Dikkat çeken"],
        "Her ne kadar": ["Ne kadar olsa da", "Her ne olursa olsun", "Olsa BILE"],
        "Dolayısıyla": ["Bu yüzden", "O nedenle", "Sonuçta"],
        "Gerçekten de": ["Sahiden", "Cidden", "Hakikaten"],
        "Başlı başına": ["Tek başına", "Kendi başına", "Salt olarak"],
        "Hemen belirtmek gerekir ki": [
            "Şunu hemen söyleyeyim",
            "Hemen ekleyeyim",
            "Bir şey eklemeliyim",
        ],
        "Düzgün bir dilde": ["Açık bir ifadeyle", "Sade bir dille", "Yalın bir anlatımla"],
        "Aşikar olanşudur ki": ["Ortada olan", "Belli olan", "Görünen"],
        "Dönüp bakıldığında": ["Geriye bakınca", "Geçmişe bakarsak", "Geçmiş düşünüldüğünde"],
        "Doğrusunu söylemek gerekirse": [
            "Açık konuşmam gerekirse",
            "Dürüst olmam gerekirse",
            "Samimi olarak",
        ],
        "Tabii ki": ["Elbette", "Doğal olarak", "Hiç kuşkusuz"],
        "Durum böyle olunca": ["Hal böyle olunca", "Bu durumda", "Böyle olunca"],
        "Büyük resme bakıldığında": ["Geniş açıdan bakarsak", "Makro açıdan", "Uzaktan bakarsak"],
        "Tuhaf olan şu ki": ["Garip olan", "Acayip olan", "İlginç olan"],
    },

    # Colloquial markers for chat/web profiles
    "colloquial_markers": [
        "bu arada",
        "aslında",
        "açıkçası",
        "kısacası",
        "basitçe söylemek gerekirse",
        "ilginç olan şu ki",
        "yalın hâliyle",
        "mesele şu ki",
        "gerçek şu ki",
        "ortaya çıktığı üzere",
        "en nihayetinde",
        "özetle",
        "her hâlükârda",
        "asıl önemli olan",
        "doğrusu",
        "hani",
        "zaten",
        "inan bana",
        "aramızda kalsın",
        "velhasıl",
        "bir düşünsen",
        "gerçekten",
        "demek istediğim",
        "yanlış anlama",
        "bu arada söyleyeyim",
        "parantez içinde",
        "tabii ki",
        "kesinlikle",
        "Allah bilir",
        "vallahi",
        "açık konuşmak gerekirse",
        "kıssadan hisse",
        "Türkçesiyle",
        "olan şu ki",
        "genel olarak",
        "abartısız",
        "hikâye şu ki",
        "mevzu şu ki",
        "pratikte",
        "bana sorarsanız",
        "bir düşünsenize",
        "hani derler ya",
        "doğru söylemek gerekirse",
        "yani şöyle diyelim",
        "kıssa dan hisse",
        "inanın bana",
        "siz düşünün",
        "bir bakmmışsınız",
        "işin gerçeği",
        "hani şey",
        "açık konuşmak lâzım",
        "meseleyi şöyle anlatayım",
        "dikkat edin",
        "bunu da belirteyim",
        "yemiş de yemiş",
        "şimdi bakın",
        "bir de şu açıdan bakalım",
        "kabul etmek lâzım",
        "hani bilirsiniz",
    ],

    # Abbreviations (don't split sentences after these)
    "abbreviations": [
        "Dr", "Prof", "Doç", "Yrd", "Öğr", "Gör",
        "Av", "Müh", "Mim", "Ecz",
        "Bşk", "Gn", "Alb", "Bnb", "Yzb", "Üçvş", "Çvş",
        "Cad", "Sok", "Mah", "Bul",
        "No", "Tel", "Faks",
        "vb", "vs", "bkz", "çev", "ed",
        "km", "cm", "mm", "kg", "mg", "ml",
        "Ltd", "Şti", "A.Ş", "T.C",
        "a.g.e", "a.g.m",
        "ör", "örn",
        "Hz", "yy",
    ],

    # Perplexity boosters (rare/unusual phrases to raise perplexity)
    "perplexity_boosters": [
        "tabiri caizse",
        "onun deyimiyle",
        "şöyle ya da böyle",
        "göründüğü kadarıyla",
        "her ne açıdan bakılırsa bakılsın",
        "en azından",
        "hakkını yemeyelim",
        "bundan bağımsız olarak",
        "söylemeye gerek yok",
        "her şeyden evvel",
        "ne olursa olsun",
        "kader bu ya",
        "hiç beklenmedik anda",
        "malumunuz",
        "beklentilerin aksine",
        "kaçınılmaz olarak",
        "hakkıyla",
        "farklı bir pencereden",
        "hengâmenin ortasında",
        "hiç kuşku yok ki",
        "bilhassa",
        "bu durum gerektiriyor ki",
        "anlaşıldığı kadarıyla",
        "olayların kalbinde",
        "gerçekçi olmak gerekirse",
        "med ve cezir arasında",
        "dönüm noktasında",
        "zaman zaman",
        "rastlantı değil",
        "aslına bakarsanız",
        "bir başka açıdan",
        "her durumda",
        "böylesi koşullar altında",
        "şaşılacak bir şey değil",
        "ironik biçimde",
        "eşit ölçüde",
        "sonuçta",
        "daha da şaşırtıcısı",
        "hassas olmak gerekirse",
        "böyle diyenler var",
        "farklılıklar göz önünde tutulursa",
        "kesin olan şudur ki",
        "kuşku götürmez biçimde",
        "hiç çekinmeden",
        "daha ince bir ifadeyle",
        "meselenin özünde",
        "genellikle söylendiği gibi",
        "ihtiyatla yaklaşırsak",
        "belirtmeden geçemeyeceğimiz",
        "abartma sayılmayacak şekilde",
        "utanıp sıkılmadan",
        "belki de rastlantı değildir ki",
        "bilinen bir gerçek olarak",
        "konunun derinine inildiğinde",
        "konuyla doğrudan bağlantılı olarak",
        "tarih boyunca göründüğü gibi",
        "nasıl olursa olsun",
        "olayların akışına bakılırsa",
        "dönüşüm sürecinde",
        "tarihsel bir perspektiften",
    ],

    # Profile metric targets
    "profile_targets": {
        "chat": {"avg_sentence_len": (6, 16), "max_sentence_len": 25},
        "web": {"avg_sentence_len": (8, 20), "max_sentence_len": 35},
        "seo": {"avg_sentence_len": (10, 22), "max_sentence_len": 40},
        "docs": {"avg_sentence_len": (10, 25), "max_sentence_len": 45},
        "formal": {"avg_sentence_len": (12, 28), "max_sentence_len": 50},
    },

    # Conjunctions for joining short sentences
    "conjunctions": ["ve", "ama", "ya da", "bu yüzden", "oysa", "üstelik", "çünkü"],

    # Split conjunctions for breaking long sentences
    "split_conjunctions": [
        ", ama", ", ve", ", bununla birlikte", ", buna rağmen",
        ", öte yandan", ", diğer taraftan", ", ancak",
        ", bu yüzden", ", ki", ". ", " — ",
        ", çünkü", ", zira", ", aksi takdirde",
        ", hatta", ", olsa bile", ", yine de",
        ", veya", ", sonra", ", önce",
        ", -dığında", ", -a kadar", ", bu arada",
        ", -dığı zaman", ", yani", ", şöyle ki",
        ", ayrıca", ", ne de", ", -den ziyade",
        ", dolayısıyla", ", böylece", ", bu nedenle",
        ", o hâlde", ", buna ek olarak", ", bunun yanı sıra",
        ", yerine", ", özellikle", ", kısacası",
        " – ",
    ],
}



"""Валидатор качества — проверка результата после обработки."""

from __future__ import annotations

import logging
import re
from difflib import SequenceMatcher

from texthumanize.analyzer import TextAnalyzer
from texthumanize.utils import AnalysisReport

logger = logging.getLogger(__name__)

class QualityValidator:
    """Проверяет качество результата после гуманизации.

    - Не удалены ли числа, ссылки, имена
    - Не пропали ли ключевые слова
    - Не превышен ли change_ratio
    - Метрики стали лучше, а не хуже
    - Структура (абзацы/списки) сохранена
    """

    def __init__(
        self,
        lang: str = "ru",
        max_change_ratio: float = 0.4,
        keep_keywords: list[str] | None = None,
    ):
        self.lang = lang
        self.max_change_ratio = max_change_ratio
        self.keep_keywords = keep_keywords or []
        self.analyzer = TextAnalyzer(lang=lang)

    def validate(
        self,
        original: str,
        processed: str,
        metrics_before: AnalysisReport | None = None,
    ) -> ValidationResult:
        """Валидировать результат обработки.

        Args:
            original: Исходный текст.
            processed: Обработанный текст.
            metrics_before: Метрики до обработки.

        Returns:
            ValidationResult с результатами проверки.
        """
        result = ValidationResult()

        # 1. Проверка change_ratio
        change_ratio = self._calc_change_ratio(original, processed)
        result.change_ratio = change_ratio
        if change_ratio > self.max_change_ratio:
            result.warnings.append(
                f"Слишком много изменений: {change_ratio:.1%} > {self.max_change_ratio:.1%}"
            )
            result.should_rollback = True

        # 2. Проверка ключевых слов
        for keyword in self.keep_keywords:
            if keyword.lower() in original.lower() and keyword.lower() not in processed.lower():
                result.errors.append(f"Потеряно ключевое слово: '{keyword}'")
                result.should_rollback = True

        # 3. Проверка числовых значений
        original_numbers = self._extract_numbers(original)
        processed_numbers = self._extract_numbers(processed)
        lost_numbers = original_numbers - processed_numbers
        if lost_numbers:
            result.warnings.append(f"Потеряны числа: {lost_numbers}")

        # 4. Проверка длины (не должна сильно измениться)
        len_ratio = len(processed) / len(original) if original else 1.0
        if len_ratio < 0.5 or len_ratio > 1.5:
            result.warnings.append(
                f"Длина текста сильно изменилась: {len_ratio:.1%}"
            )

        # 5. Проверка сохранения структуры (абзацы/строки)
        self._check_structure_preservation(original, processed, result)

        # 6. Проверка метрик (если есть до-обработка)
        if metrics_before:
            metrics_after = self.analyzer.analyze(processed)
            result.metrics_before = metrics_before
            result.metrics_after = metrics_after

            # Искусственность должна снизиться или не измениться
            diff = metrics_after.artificiality_score - metrics_before.artificiality_score
            if diff > 15:
                # Серьёзное ухудшение — откат
                result.errors.append(
                    f"Искусственность значительно выросла: "
                    f"{metrics_before.artificiality_score:.1f} → "
                    f"{metrics_after.artificiality_score:.1f} (+{diff:.1f})"
                )
                result.should_rollback = True
            elif diff > 5:
                result.warnings.append(
                    "Оценка искусственности увеличилась после обработки"
                )

        result.is_valid = not result.errors and not result.should_rollback
        return result

    def _check_structure_preservation(
        self,
        original: str,
        processed: str,
        result: ValidationResult,
    ) -> None:
        """Проверить, что абзацная/списковая структура сохранена."""
        orig_lines = [ln for ln in original.split('\n') if ln.strip()]
        proc_lines = [ln for ln in processed.split('\n') if ln.strip()]

        # Количество непустых строк не должно уменьшиться более чем на 30%
        if orig_lines and proc_lines:
            ratio = len(proc_lines) / len(orig_lines)
            if ratio < 0.7:
                result.warnings.append(
                    f"Потеряна структура: строк было {len(orig_lines)}, "
                    f"стало {len(proc_lines)}"
                )

        # Количество абзацев (блоков, разделённых пустыми строками)
        orig_paras = len(re.split(r'\n\s*\n', original))
        proc_paras = len(re.split(r'\n\s*\n', processed))
        if orig_paras > 1 and proc_paras < orig_paras:
            result.warnings.append(
                f"Абзацы объединены: было {orig_paras}, стало {proc_paras}"
            )

    def _calc_change_ratio(self, original: str, processed: str) -> float:
        """Вычислить долю изменений (SequenceMatcher-based)."""
        if not original:
            return 0.0
        orig_words = original.split()
        proc_words = processed.split()
        if not orig_words:
            return 0.0
        matcher = SequenceMatcher(None, orig_words, proc_words)
        return min(1.0 - matcher.ratio(), 1.0)

    def _extract_numbers(self, text: str) -> set[str]:
        """Извлечь числовые значения из текста."""
        return set(re.findall(r'\b\d+(?:[.,]\d+)?\b', text))


class ValidationResult:
    """Результат валидации."""

    def __init__(self) -> None:
        self.is_valid: bool = True
        self.should_rollback: bool = False
        self.change_ratio: float = 0.0
        self.errors: list[str] = []
        self.warnings: list[str] = []
        self.metrics_before: AnalysisReport | None = None
        self.metrics_after: AnalysisReport | None = None

    def __repr__(self) -> str:
        status = "VALID" if self.is_valid else "INVALID"
        return (
            f"ValidationResult({status}, "
            f"change_ratio={self.change_ratio:.1%}, "
            f"errors={len(self.errors)}, warnings={len(self.warnings)})"
        )



"""Пайплайн обработки текста — оркестрация всех этапов."""

from __future__ import annotations

import logging
import os
import time
from difflib import SequenceMatcher
from typing import Callable, Protocol

from texthumanize.analyzer import TextAnalyzer
from texthumanize.cjk_segmenter import CJKSegmenter, is_cjk_text
from texthumanize.coherence_repair import CoherenceRepairer
from texthumanize.content_classifier import ContentType
from texthumanize.content_classifier import classify as classify_content
from texthumanize.decancel import Debureaucratizer
from texthumanize.fingerprint_randomizer import FingerprintRandomizer
from texthumanize.grammar_fix import GrammarCorrector
from texthumanize.lang import get_language_tier
from texthumanize.liveliness import LivelinessInjector
from texthumanize.naturalizer import TextNaturalizer
from texthumanize.normalizer import TypographyNormalizer
from texthumanize.paraphraser_ext import SemanticParaphraser
from texthumanize.readability_opt import ReadabilityOptimizer
from texthumanize.repetitions import RepetitionReducer
from texthumanize.segmenter import Segmenter
from texthumanize.sentence_validator import SentenceValidator
from texthumanize.structure import StructureDiversifier
from texthumanize.stylistic import StylisticAnalyzer, StylisticFingerprint
from texthumanize.syntax_rewriter import SyntaxRewriter
from texthumanize.tone_harmonizer import ToneHarmonizer
from texthumanize.universal import UniversalProcessor
from texthumanize.utils import AnalysisReport, HumanizeOptions, HumanizeResult
from texthumanize.validator import QualityValidator
from texthumanize.watermark import WatermarkDetector
from texthumanize.word_lm import WordLanguageModel

logger = logging.getLogger(__name__)

class StagePlugin(Protocol):
    """Protocol for custom pipeline stage plugins."""

    def process(self, text: str, lang: str, profile: str, intensity: int) -> str:
        """Process text and return the modified version."""
        ...

# Тип хука: функция (text, lang) -> text
HookFn = Callable[[str, str], str]

class Pipeline:
    """Оркестратор пайплайна гуманизации текста.

    Этапы обработки:
    0. Очистка водяных знаков (zero-width, homoglyphs, invisible Unicode)
    1. Сегментация (защита кода, URL, email и т.д.)
    2. Нормализация типографики
    3. Деканцеляризация (для языков с полным словарём)
    4. Разнообразие структуры (для языков с полным словарём)
    5. Уменьшение повторов (для языков с полным словарём)
    6. Инъекция «живости» (для языков с полным словарём)
    7. Семантическое перефразирование (синтаксические трансформации)
    8. Гармонизация тона (приведение к стилю профиля)
    9. Универсальная обработка (для ВСЕХ языков)
    10. Натурализация стиля (для ВСЕХ языков — КЛЮЧЕВОЙ ЭТАП)
    11. Оптимизация читаемости (разбивка/объединение предложений)
    12. Грамматическая коррекция (финальная полировка)
    13. Коррекция когерентности (связность абзацев)
    14. Валидация качества
    15. Восстановление защищённых сегментов

    Supports custom plugins that can be inserted before or after
    any built-in stage via register_plugin().
    """

    # Class-level default plugin registry (guarded by _cls_lock)
    _cls_lock = __import__('threading').Lock()
    _class_plugins_before: dict[str, list[StagePlugin]] = {}
    _class_plugins_after: dict[str, list[StagePlugin]] = {}
    _class_hooks_before: dict[str, list[HookFn]] = {}
    _class_hooks_after: dict[str, list[HookFn]] = {}

    STAGE_NAMES = (
        "watermark", "segmentation", "typography", "debureaucratization",
        "structure", "repetitions", "liveliness",
        "paraphrasing", "syntax_rewriting", "tone", "universal", "naturalization",
        "paraphrase_engine", "sentence_restructuring",
        "entropy_injection",
        "readability", "grammar", "coherence",
        "validation", "restore",
    )

    def __init__(self, options: HumanizeOptions | None = None):
        self.options = options or HumanizeOptions()
        # Instance-level copies of class plugins (isolation between instances)
        self._plugins_before = {k: list(v) for k, v in self._class_plugins_before.items()}
        self._plugins_after = {k: list(v) for k, v in self._class_plugins_after.items()}
        self._hooks_before = {k: list(v) for k, v in self._class_hooks_before.items()}
        self._hooks_after = {k: list(v) for k, v in self._class_hooks_after.items()}

    # ─── Plugin API ───────────────────────────────────────────

    @classmethod
    def register_plugin(
        cls,
        plugin: StagePlugin,
        *,
        before: str | None = None,
        after: str | None = None,
    ) -> None:
        """Register a custom pipeline stage plugin.

        Args:
            plugin: Object implementing the StagePlugin protocol (needs a
                    ``process(text, lang, profile, intensity) -> str`` method).
            before: Insert this plugin *before* the named stage.
            after: Insert this plugin *after* the named stage.

        Raises:
            ValueError: If neither ``before`` nor ``after`` is given, or
                        if the stage name is unknown.
        """
        if before is None and after is None:
            raise ValueError("Specify 'before' or 'after' stage name.")
        target = before or after
        if target not in cls.STAGE_NAMES:
            raise ValueError(
                f"Unknown stage: {target}. Valid stages: {cls.STAGE_NAMES}"
            )
        with cls._cls_lock:
            if before:
                cls._class_plugins_before.setdefault(before, []).append(plugin)
            else:
                cls._class_plugins_after.setdefault(after, []).append(plugin)  # type: ignore[arg-type]

    @classmethod
    def register_hook(
        cls,
        hook: HookFn,
        *,
        before: str | None = None,
        after: str | None = None,
    ) -> None:
        """Register a lightweight hook function.

        Args:
            hook: Callable ``(text: str, lang: str) -> str``.
            before: Run the hook *before* the named stage.
            after: Run the hook *after* the named stage.
        """
        if before is None and after is None:
            raise ValueError("Specify 'before' or 'after' stage name.")
        target = before or after
        if target not in cls.STAGE_NAMES:
            raise ValueError(
                f"Unknown stage: {target}. Valid stages: {cls.STAGE_NAMES}"
            )
        if before:
            cls._class_hooks_before.setdefault(before, []).append(hook)
        else:
            cls._class_hooks_after.setdefault(after, []).append(hook)  # type: ignore[arg-type]

    @classmethod
    def clear_plugins(cls) -> None:
        """Remove all registered plugins and hooks."""
        with cls._cls_lock:
            cls._class_plugins_before.clear()
            cls._class_plugins_after.clear()
            cls._class_hooks_before.clear()
            cls._class_hooks_after.clear()

    def _run_plugins(self, stage: str, text: str, lang: str, *, is_before: bool) -> str:
        """Run registered plugins/hooks for a stage."""
        registry = self._plugins_before if is_before else self._plugins_after
        hooks_reg = self._hooks_before if is_before else self._hooks_after

        for plugin in registry.get(stage, []):
            text = plugin.process(
                text, lang, self.options.profile, self.options.intensity
            )
        for hook in hooks_reg.get(stage, []):
            text = hook(text, lang)
        return text

    def _apply_custom_dict(self, text: str) -> tuple[str, list[dict]]:
        """Apply user-supplied custom_dict replacements.

        Supports both ``{"word": "replacement"}`` and
        ``{"word": ["var1", "var2"]}`` formats.  When a list is given,
        a random variant is chosen (respecting the pipeline seed).

        Returns:
            Tuple of (modified text, list of change dicts).
        """
        import random as _rnd
        import re as _re

        changes: list[dict] = []
        rng = _rnd.Random(self.options.seed)
        for pattern, replacement in self.options.custom_dict.items():  # type: ignore[union-attr]
            if isinstance(replacement, list):
                if not replacement:
                    continue
                chosen = rng.choice(replacement)
            else:
                chosen = replacement
            # Whole-word, case-insensitive match
            escaped = _re.escape(pattern)
            regex = _re.compile(rf"\b{escaped}\b", _re.IGNORECASE)
            if regex.search(text):
                text = regex.sub(chosen, text)
                changes.append({
                    "type": "custom_dict",
                    "description": f"custom_dict: «{pattern}» → «{chosen}»",
                })
        return text, changes

    # Maximum allowed time for a single pipeline run (seconds).
    # Can be overridden via TEXTHUMANIZE_TIMEOUT env var (useful for CI with coverage).
    PIPELINE_TIMEOUT: float = float(os.environ.get("TEXTHUMANIZE_TIMEOUT", "30"))

    def run(self, text: str, lang: str) -> HumanizeResult:
        """Запустить пайплайн обработки.

        Args:
            text: Текст для обработки.
            lang: Код языка.

        Returns:
            HumanizeResult с обработанным текстом и метаданными.

        Raises:
            TimeoutError: If processing exceeds PIPELINE_TIMEOUT seconds.
        """

        # Adaptive max_change_ratio: higher intensity allows more changes
        # intensity 30 → 0.42, intensity 50 → 0.50, intensity 80 → 0.62
        _user_max = self.options.constraints.get("max_change_ratio", None)
        if _user_max is not None:
            max_change = _user_max
        else:
            _i = self.options.intensity / 100.0
            max_change = min(0.80, 0.30 + _i * 0.50)
        deadline = time.monotonic() + self.PIPELINE_TIMEOUT

        def _check_deadline() -> None:
            if time.monotonic() > deadline:
                raise TimeoutError(
                    f"Pipeline processing exceeded {self.PIPELINE_TIMEOUT}s timeout"
                )

        self._check_deadline = _check_deadline

        result = self._run_pipeline(text, lang, intensity_factor=1.0)
        _check_deadline()

        # Graduated retry: если change_ratio слишком высокий,
        # повторяем с пониженной интенсивностью
        if result.change_ratio > max_change:
            for factor in (0.4, 0.20, 0.10, 0.05):
                _check_deadline()
                retry = self._run_pipeline(text, lang, intensity_factor=factor)
                if retry.change_ratio <= max_change:
                    result = retry
                    break
                if retry.quality_score > result.quality_score:
                    result = retry

        # ── Per-run detection cache ────────────────────────────
        # Avoids recomputing detect_ai() for the same text within a single
        # run() invocation. The regression guard often re-detects text that
        # was already scored in the detector-in-the-loop.
        # Use text itself as key (not hash()) to avoid hash collisions.
        _detect_cache: dict[str, dict] = {}

        def _cached_detect(txt: str, *, lang: str) -> dict:  # type: ignore[type-arg]
            """Detect AI with per-run memoization.

            Uses fast MLP-only detection for the in-the-loop passes
            (~3-5x faster than full detection). Full detection is used
            only for the very first call to get accurate baseline.
            """
            cached = _detect_cache.get(txt)
            if cached is not None:
                return cached
            if len(_detect_cache) == 0:
                # First call: full detection for accurate baseline
                from texthumanize.core import detect_ai as _full_detect
                result_d: dict = _full_detect(txt, lang=lang)  # type: ignore[assignment]
            else:
                # Subsequent calls (in-the-loop): fast MLP-only
                from texthumanize.core import detect_ai_fast as _fast_detect
                result_d = _fast_detect(txt, lang=lang)
            _detect_cache[txt] = result_d
            return result_d

        # ── Detector-in-the-loop ──────────────────────────────
        # After humanization, check if the AI detector still flags
        # the result. If so, run another pass with escalated intensity
        # on the already-processed text to push it further from AI.
        target_ai = self.options.constraints.get("target_ai_score", 0.20)
        max_loops = self.options.constraints.get("max_detection_loops", 3)
        ai_before = result.metrics_before.get("artificiality_score", 0)

        # Also check the full combined detection score on the original text.
        # The heuristic analyzer (artificiality_score) may underestimate AI
        # probability for texts the neural MLP detector catches. Use the
        # higher of the two signals to decide whether to loop.
        _full_before = _cached_detect(text, lang=lang).get("combined_score", 0.0)
        _trigger = ai_before > 40 or _full_before > 0.50

        # Only loop if original text was actually AI-like
        if max_loops > 0 and _trigger:
            best_result = result
            detect_result = _cached_detect(result.text, lang=lang)
            best_score = detect_result.get("combined_score", 1.0)

            for loop_i in range(max_loops):
                _check_deadline()

                if best_score <= target_ai:
                    break  # Successfully humanized below threshold

                # Calculate change from original — scale headroom with intensity
                # to prevent inverse monotonicity (low intensity getting more
                # total change than high intensity via the loop)
                loop_change = self._calc_change_ratio(text, best_result.text)
                # When user explicitly sets max_change_ratio, use tight
                # headroom (5%) to respect their constraint.
                if _user_max is not None:
                    _max_total = max_change + 0.05
                else:
                    _headroom = max(0.10, (self.options.intensity / 100) * 0.25)
                    _max_total = max_change + _headroom

                # If AI score is still very high (>0.60) and the user hasn't
                # set an explicit max_change_ratio, allow much more change —
                # the text NEEDS aggressive editing to evade.
                if best_score > 0.60 and _user_max is None:
                    _max_total = max(0.80, _max_total)

                if loop_change > _max_total:
                    break  # Already changed enough, don't risk quality

                # Targeted escalation: analyze which detector features
                # are still flagging AI, and escalate accordingly.
                # Gentler escalation to avoid over-processing cliff.
                escalation = 1.10 + 0.15 * loop_i  # 1.10×, 1.25×, 1.40×
                esc_intensity = min(92, int(self.options.intensity * escalation))

                # Use detection details to target weak features
                loop_profile = self.options.profile
                detect_details = detect_result.get("details", {})
                neural_score = detect_details.get("neural_score", best_score)

                # If neural score is high, focus on structural transforms
                # by using a higher-intensity profile
                if neural_score > 0.6:
                    esc_intensity = min(esc_intensity + 10, 98)

                loop_opts = HumanizeOptions(
                    lang=self.options.lang,
                    profile=loop_profile,
                    intensity=esc_intensity,
                    preserve=dict(self.options.preserve),
                    constraints={
                        **self.options.constraints,
                        "max_change_ratio": 0.40,  # Per-loop limit
                    },
                    seed=(self.options.seed or 42) + loop_i + 1,
                )
                loop_pipeline = Pipeline(loop_opts)
                loop_pipeline._check_deadline = _check_deadline

                try:
                    loop_result = loop_pipeline._run_pipeline(
                        best_result.text, lang, intensity_factor=1.0,
                    )
                except (TimeoutError, Exception):
                    break

                # Accept loop result ONLY if it improves full detection score
                detect_result = _cached_detect(loop_result.text, lang=lang)
                loop_score = detect_result.get("combined_score", 1.0)
                if loop_score < best_score:
                    total_change = self._calc_change_ratio(text, loop_result.text)
                    if total_change <= _max_total:
                        best_result = HumanizeResult(
                            original=text,
                            text=loop_result.text,
                            lang=lang,
                            profile=self.options.profile,
                            intensity=self.options.intensity,
                            changes=[*best_result.changes, {"type": "detection_loop", "description": f"Loop {loop_i + 1}: AI " f"{best_score:.0%}→{loop_score:.0%}"}, *loop_result.changes],
                            metrics_before=best_result.metrics_before,
                            metrics_after=loop_result.metrics_after,
                        )
                        best_score = loop_score

            # If user set an explicit max_change_ratio constraint,
            # reject loop results that violate it. Fall back to the
            # pre-loop result which respects the constraint.
            if _user_max is not None:
                loop_change = self._calc_change_ratio(text, best_result.text)
                if loop_change > _user_max + 0.05:
                    # Detection loop overshot the constraint; keep pre-loop
                    pass  # result is still the pre-loop one captured above
                else:
                    result = best_result
            else:
                result = best_result

        # ── LLM-assisted evasion ─────────────────────────────
        # When an OpenAI API key is provided and the AI score is still
        # above the target after local transforms, use the LLM to
        # rewrite the text at sentence level for deeper evasion.
        _api_key = self.options.openai_api_key
        if _api_key:
            try:
                _check_deadline()
                llm_score = _cached_detect(result.text, lang=lang).get(
                    "combined_score", 0.0,
                )
                if llm_score > target_ai:
                    llm_result = self._llm_assisted_rewrite(
                        original=text,
                        current=result,
                        lang=lang,
                        api_key=_api_key,
                        model=self.options.openai_model,
                        target_score=target_ai,
                        max_change=max_change,
                        detect_fn=_cached_detect,
                        check_deadline=_check_deadline,
                    )
                    if llm_result is not None:
                        result = llm_result
            except (TimeoutError, Exception):
                pass  # LLM evasion is advisory, never blocks return

        # ── Regression guard ─────────────────────────────────
        # If humanization made the AI score WORSE (increased), try
        # graduated fallback with decreasing intensity factors.
        # This replaces the old binary 0.05× fallback which was too
        # aggressive and returned near-original (high AI) text.
        try:
            score_after = _cached_detect(result.text, lang=lang).get(
                "combined_score", 0.0,
            )
            # Compare like-for-like: full ensemble before vs. after
            score_before_full = _cached_detect(text, lang=lang).get(
                "combined_score", 0.0,
            )
            if score_after > score_before_full + 0.01:  # Worsened by >1%
                # Graduated fallback: try decreasing intensity factors,
                # keep the best result (lowest AI score)
                best_fallback = result
                best_fb_score = score_after
                for fb_factor in (0.5, 0.3, 0.15, 0.08):
                    _check_deadline()
                    fb_result = self._run_pipeline(
                        text, lang, intensity_factor=fb_factor,
                    )
                    fb_score = _cached_detect(fb_result.text, lang=lang).get(
                        "combined_score", 0.0,
                    )
                    if fb_score < best_fb_score:
                        best_fallback = fb_result
                        best_fb_score = fb_score
                    # If we found something better than original, stop
                    if fb_score <= score_before_full:
                        break
                result = best_fallback
                result.changes.append({
                    "type": "regression_guard",
                    "description": (
                        f"Откат: AI {score_before_full:.0%}→{score_after:.0%} "
                        f"(graduated fallback → {best_fb_score:.0%})"
                    ),
                })
        except Exception:
            pass  # Guard is advisory, never blocks return

        # ── Hard constraint enforcement ────────────────────────
        # If the user set max_change_ratio explicitly, guarantee the final
        # result respects it (within 5% tolerance). Re-run at progressively
        # lower intensity if necessary.
        if _user_max is not None and result.change_ratio > _user_max + 0.05:
            for fb_factor in (0.3, 0.15, 0.08, 0.03):
                _check_deadline()
                fb = self._run_pipeline(text, lang, intensity_factor=fb_factor)
                if fb.change_ratio <= _user_max + 0.05:
                    result = fb
                    break

        # ── Final sentence-level sanitization ─────────────────
        # After all pipeline passes (graduated retry, detection loops,
        # regression guard), clean up any remaining artifacts that
        # were introduced by late passes or cross-pass interactions.
        import re as _re_final
        _final_text = result.text
        #  Double conjunctions: "and and", "и и"
        _final_text = _re_final.sub(
            r'\b(and|but|or|yet|so|и|і|а|але|но|und|oder|aber'
            r'|et|ou|mais|y|o|pero)\s+\1\b',
            r'\1', _final_text, flags=_re_final.IGNORECASE,
        )
        #  Dangling conjunction before punctuation: "and." "and,"
        _final_text = _re_final.sub(
            r',?\s*\b(and|but|or|и|і|а|але|но)\b\s*([.!?;])',
            r'\2', _final_text, flags=_re_final.IGNORECASE,
        )
        #  Conjunction chain residue: ", and, but,"
        _final_text = _re_final.sub(
            r'(?:,\s*\b(?:and|but|or|however|moreover|also)\b\s*){2,}',
            ', ', _final_text, flags=_re_final.IGNORECASE,
        )
        if _final_text != result.text:
            result = HumanizeResult(
                original=result.original,
                text=_final_text,
                lang=result.lang,
                profile=result.profile,
                intensity=result.intensity,
                changes=[*result.changes, {
                    "type": "final_sanitization",
                    "description": "Финальная очистка артефактов",
                }],
                metrics_before=result.metrics_before,
                metrics_after=result.metrics_after,
            )

        return result

    # ── LLM-assisted rewrite helper ─────────────────────────

    _LLM_SYSTEM_PROMPT = (
        "You are a text rewriter. Rewrite the following text to sound "
        "completely natural and human-written in {lang}. "
        "CRITICAL RULES:\n"
        "- Use short, simple words (1-2 syllables preferred)\n"
        "- Vary sentence lengths dramatically: mix very short (3-5 words) "
        "with medium (10-15 words) sentences\n"
        "- Add minor imperfections: contractions, informal transitions, "
        "occasional fragments\n"
        "- Avoid formal/academic vocabulary\n"
        "- Start sentences with different words\n"
        "- Preserve the original meaning\n"
        "Only output the rewritten text, nothing else."
    )

    def _llm_assisted_rewrite(
        self,
        original: str,
        current: HumanizeResult,
        lang: str,
        api_key: str,
        model: str,
        target_score: float,
        max_change: float,
        detect_fn: Callable[..., dict],
        check_deadline: Callable[[], None],
    ) -> HumanizeResult | None:
        """Use OpenAI LLM to rewrite text that local transforms couldn't evade.

        Returns an improved HumanizeResult or None if LLM didn't help.
        """
        from texthumanize.ai_backend import AIBackend

        check_deadline()

        try:
            ai = AIBackend(
                openai_api_key=api_key,
                openai_model=model,
                prefer="openai",
            )
            # Rewrite the already-humanized text for deeper evasion
            rewritten = ai.paraphrase(
                current.text, lang=lang, style=self.options.profile,
            )
        except Exception:
            return None

        if not rewritten or not rewritten.strip():
            return None

        # Also try improve_naturalness for a second variant
        try:
            check_deadline()
            natural = ai.improve_naturalness(current.text, lang=lang)
        except Exception:
            natural = None

        # Evaluate both variants, pick the best
        candidates = [rewritten]
        if natural and natural.strip() and natural != rewritten:
            candidates.append(natural)

        best_text: str | None = None
        pre_score = detect_fn(current.text, lang=lang).get("combined_score", 1.0)
        best_score = pre_score

        for candidate in candidates:
            check_deadline()
            try:
                cand_detect = detect_fn(candidate, lang=lang)
            except Exception:
                continue
            cand_score = cand_detect.get("combined_score", 1.0)
            if cand_score < best_score:
                best_text = candidate
                best_score = cand_score

        if best_text is None:
            return None

        # Build result
        return HumanizeResult(
            original=original,
            text=best_text,
            lang=lang,
            profile=self.options.profile,
            intensity=self.options.intensity,
            changes=[
                *current.changes,
                {
                    "type": "llm_evasion",
                    "description": (
                        f"LLM rewrite ({model}): "
                        f"AI {pre_score:.0%}→{best_score:.0%}"
                    ),
                },
            ],
            metrics_before=current.metrics_before,
            metrics_after=current.metrics_after,
        )

    @staticmethod
    def _calc_change_ratio(original: str, current: str) -> float:
        """Вычислить текущий change_ratio (SequenceMatcher-based)."""
        if not original:
            return 0.0
        orig_words = original.split()
        curr_words = current.split()
        if not orig_words:
            return 0.0
        matcher = SequenceMatcher(None, orig_words, curr_words)
        return min(1.0 - matcher.ratio(), 1.0)

    def _typography_only(
        self,
        text: str,
        lang: str,
        metrics_before: AnalysisReport,
        changes: list[dict],
        *,
        preserve_config: dict | None = None,
    ) -> HumanizeResult:
        """Fast path for already-natural text: only typography normalization.

        Skips all semantic stages to prevent over-processing genuine
        human-written content.
        """
        original = text
        all_changes = list(changes)
        all_changes.append({
            "type": "skip_natural",
            "description": (
                f"Текст уже естественный (AI={metrics_before.artificiality_score:.0f}%). "
                "Применяется только типографика."
            ),
        })

        # Watermark cleaning (even for natural text)
        wm_detector = WatermarkDetector(lang=lang)
        wm_report = wm_detector.detect(text)
        if wm_report.has_watermarks:
            text = wm_report.cleaned_text
            all_changes.append({
                "type": "watermark_cleaning",
                "description": (
                    f"Водяные знаки: {', '.join(wm_report.watermark_types)} "
                    f"(удалено {wm_report.characters_removed} символов)"
                ),
            })

        # Segmentation
        preserve = preserve_config or {}
        segmenter = Segmenter(preserve=preserve)
        segmented = segmenter.segment(text)
        text = segmented.text

        # Typography only
        normalizer = TypographyNormalizer(
            profile=self.options.profile,
            lang=lang,
        )
        text = normalizer.normalize(text)
        all_changes.extend(normalizer.changes)

        # Restore segments
        text = segmented.restore(text)

        # Metrics after
        analyzer = TextAnalyzer(lang=lang)
        metrics_after = analyzer.analyze(text)

        return HumanizeResult(
            original=original,
            text=text,
            lang=lang,
            profile=self.options.profile,
            intensity=self.options.intensity,
            changes=all_changes,
            metrics_before={
                "artificiality_score": metrics_before.artificiality_score,
                "avg_sentence_length": metrics_before.avg_sentence_length,
                "bureaucratic_ratio": metrics_before.bureaucratic_ratio,
                "connector_ratio": metrics_before.connector_ratio,
                "repetition_score": metrics_before.repetition_score,
                "typography_score": metrics_before.typography_score,
                "predictability_score": metrics_before.predictability_score,
                "vocabulary_richness": metrics_before.vocabulary_richness,
            },
            metrics_after={
                "artificiality_score": metrics_after.artificiality_score,
                "avg_sentence_length": metrics_after.avg_sentence_length,
                "bureaucratic_ratio": metrics_after.bureaucratic_ratio,
                "connector_ratio": metrics_after.connector_ratio,
                "repetition_score": metrics_after.repetition_score,
                "typography_score": metrics_after.typography_score,
                "predictability_score": metrics_after.predictability_score,
                "vocabulary_richness": metrics_after.vocabulary_richness,
            },
        )

    def _safe_stage(
        self,
        stage_name: str,
        text: str,
        lang: str,
        fn: Callable[[], tuple[str, list[dict]]],
        stage_timings: dict[str, float],
    ) -> tuple[str, list[dict]]:
        """Execute a pipeline stage with error isolation and profiling.

        If the stage raises an exception, returns the original text
        unchanged and records a skip change.
        """
        t0 = time.perf_counter()
        try:
            new_text, changes = fn()
            stage_timings[stage_name] = time.perf_counter() - t0
            return new_text, changes
        except Exception as exc:
            stage_timings[stage_name] = time.perf_counter() - t0
            logging.getLogger("texthumanize").warning(
                "Stage '%s' failed: %s — skipping", stage_name, exc,
            )
            return text, [{
                "type": "stage_skipped",
                "description": f"Этап «{stage_name}» пропущен из-за ошибки: {exc}",
            }]

    @staticmethod
    def _strip_fragment_chains(text: str, lang: str) -> str:
        """Remove over-dense short-fragment chains injected by multiple stages.

        Handles two types of artifacts:
        1. Consecutive short fragments: "Here's the thing. But why? Actually."
        2. In-sentence garbled chains: ", and, but, however, and..."

        Preserves paragraph boundaries (\\n\\n).
        """
        import re as _re

        # Process each paragraph independently to preserve boundaries.
        paragraphs = text.split('\n\n')
        cleaned_paras: list[str] = []

        for para in paragraphs:
            if not para.strip():
                cleaned_paras.append(para)
                continue

            # Pass 0: Clean garbled conjunction/connector chains within sentences.
            para = _re.sub(
                r'(?:,\s*(?:and|but|or|yet|so|however|granted|moreover|furthermore|also|thus|hence)'
                r'(?:\s*,\s*|\s+)){2,}',
                ', ', para, flags=_re.IGNORECASE,
            )
            # Russian/Ukrainian connector stacking
            para = _re.sub(
                r'(?:,?\s*(?:и|а|но|однако|также|ещё|ще|причём|причому|крім того|кроме того)'
                r'(?:\s*,\s*|\s+)){2,}',
                ', ', para, flags=_re.IGNORECASE,
            )
            # Double conjunctions: "и и", "and and"
            para = _re.sub(r'\b(и|і|and|und|et|y|e)\s+\1\b', r'\1', para, flags=_re.IGNORECASE)

            # Pass 1: Split into sentences and remove consecutive short fragments.
            parts = _re.split(r'(?<=[.!?])\s+(?=[A-ZА-ЯІЇЄҐ])', para)
            if len(parts) < 4:
                cleaned_paras.append(para)
                continue

            def _is_fragment(s: str) -> bool:
                words = s.split()
                return len(words) <= 5 and bool(s.strip())

            # Strip consecutive fragments (keep first in each run)
            cleaned: list[str] = []
            prev_was_fragment = False
            for s in parts:
                if _is_fragment(s):
                    if prev_was_fragment:
                        continue  # skip consecutive fragment
                    prev_was_fragment = True
                else:
                    prev_was_fragment = False
                cleaned.append(s)

            # Enforce max 1 fragment per 5-sentence window
            window = 5
            result: list[str] = []
            frag_count_in_window = 0
            for i, s in enumerate(cleaned):
                if _is_fragment(s):
                    frag_count_in_window += 1
                    if frag_count_in_window > 1:
                        continue  # skip excess fragment in this window
                if i > 0 and i % window == 0:
                    frag_count_in_window = 0
                result.append(s)

            cleaned_paras.append(' '.join(result))

        return '\n\n'.join(cleaned_paras)

    def _run_pipeline(
        self, text: str, lang: str, *, intensity_factor: float = 1.0,
    ) -> HumanizeResult:
        """Выполнить один проход пайплайна.

        Args:
            text: Текст для обработки.
            lang: Код языка.
            intensity_factor: Множитель интенсивности (0-1) для graduated retry.
        """
        original = text
        all_changes: list[dict] = []
        stage_timings: dict[str, float] = {}
        checkpoints: list[tuple[str, str]] = []  # (stage_name, text_after_stage)

        # ── Sentence-level integrity validator ────────────────
        # Catches broken sentences BETWEEN stages (not just at the end).
        # After each major transformation, compares output sentences
        # against their pre-stage versions and reverts broken ones.
        _sv = SentenceValidator(lang=lang)

        # Анализ до обработки
        analyzer = TextAnalyzer(lang=lang)
        metrics_before = analyzer.analyze(text)

        # ── Stage 0: Content type classification ──────────────
        _t0 = time.perf_counter()
        content_profile = classify_content(text, lang=lang)
        stage_timings["content_classify"] = time.perf_counter() - _t0
        all_changes.append({
            "type": "content_classification",
            "description": (
                f"Тип контента: {content_profile.content_type.value} "
                f"(уверенность {content_profile.confidence:.0%})"
            ),
        })

        # ── Адаптивная интенсивность ──────────────────────────
        # Автоматически корректируем intensity на основе artificiality_score:
        # - Высокий AI-скор (>60) → усиливаем обработку
        # - Низкий AI-скор (<25) → мягко ослабляем, но гарантируем не менее
        #   50% от запрошенной intensity (монотонность)
        effective_options = self.options
        ai_score = metrics_before.artificiality_score
        base_intensity = self.options.intensity

        # Применяем graduated retry factor
        base_intensity = max(5, int(base_intensity * intensity_factor))

        # Cap base intensity at 92 to allow strong single-pass processing.
        # The detector-in-the-loop provides additional passes if needed.
        base_intensity = min(base_intensity, 92)

        # Content-type intensity cap — protects sensitive content
        base_intensity = min(base_intensity, content_profile.max_intensity_cap)

        if ai_score >= 70:
            # Сильно «искусственный» текст — aggressive boost
            adjusted = min(95, int(base_intensity * 1.30))
        elif ai_score >= 50:
            # Средне «искусственный» — moderate boost
            adjusted = min(90, int(base_intensity * 1.20))
        elif ai_score <= 5:
            # Полностью «живой» текст — применяем только типографику
            return self._typography_only(
                text, lang, metrics_before, all_changes,
                preserve_config=dict(self.options.preserve),
            )
        elif ai_score <= 10:
            # Почти полностью «живой» текст — минимальная обработка
            adjusted = max(5, int(base_intensity * 0.2))
        elif ai_score <= 15:
            # Почти «живой» текст — сильно ослабляем
            adjusted = max(8, int(base_intensity * 0.35))
        elif ai_score <= 25:
            # Слабо «искусственный» — ослабляем
            adjusted = max(10, int(base_intensity * 0.5))
        else:
            adjusted = base_intensity

        if adjusted != self.options.intensity:
            # Создаём копию опций с адаптированной интенсивностью
            # (either intensity_factor changed it OR ai_score adjustment)
            effective_options = HumanizeOptions(
                lang=self.options.lang,
                profile=self.options.profile,
                intensity=adjusted,
                preserve=dict(self.options.preserve),
                constraints=dict(self.options.constraints),
                seed=self.options.seed,
            )
            all_changes.append({
                "type": "adaptive_intensity",
                "description": (
                    f"Адаптация: AI-скор={ai_score:.0f}%, "
                    f"intensity {base_intensity}→{adjusted}"
                ),
            })

        # 1. Сегментация — защита неизменяемых блоков
        preserve_config = dict(self.options.preserve)

        # ── 0. Очистка водяных знаков ─────────────────────────
        _t0 = time.perf_counter()
        text = self._run_plugins("watermark", text, lang, is_before=True)
        wm_detector = WatermarkDetector(lang=lang)
        wm_report = wm_detector.detect(text)
        if wm_report.has_watermarks:
            text = wm_report.cleaned_text
            all_changes.append({
                "type": "watermark_cleaning",
                "description": (
                    f"Водяные знаки: {', '.join(wm_report.watermark_types)} "
                    f"(удалено {wm_report.characters_removed} символов, "
                    f"уверенность {wm_report.confidence:.0%})"
                ),
            })
        text = self._run_plugins("watermark", text, lang, is_before=False)
        stage_timings["watermark"] = time.perf_counter() - _t0

        # ── Стилистический отпечаток ──────────────────────────
        # Если задан target_style, анализируем текущий стиль
        # и корректируем параметры для приближения к целевому
        style_meta: dict = {}
        target_fp = self.options.target_style
        # Resolve preset name → StylisticFingerprint
        if isinstance(target_fp, str):
            from texthumanize.stylistic import STYLE_PRESETS
            target_fp = STYLE_PRESETS.get(target_fp)
        if target_fp is not None and isinstance(target_fp, StylisticFingerprint):
            style_analyzer = StylisticAnalyzer(lang=lang)
            source_fp = style_analyzer.extract(text)
            style_similarity = source_fp.similarity(target_fp)
            style_meta = {
                "style_similarity_before": round(style_similarity, 3),
                "target_sentence_mean": round(target_fp.sentence_length_mean, 1),
                "source_sentence_mean": round(source_fp.sentence_length_mean, 1),
            }
            all_changes.append({
                "type": "style_matching",
                "description": (
                    f"Стилистическое сходство: {style_similarity:.1%}. "
                    f"Целевая длина предложений: {target_fp.sentence_length_mean:.0f} слов"
                ),
            })
        # Добавляем keep_keywords в protect
        keep_kw = self.options.constraints.get("keep_keywords", [])
        if keep_kw:
            preserve_config.setdefault("keep_keywords", [])
            preserve_config["keep_keywords"] = keep_kw

        _t0 = time.perf_counter()
        segmenter = Segmenter(preserve=preserve_config)
        segmented = segmenter.segment(text)
        text = segmented.text
        stage_timings["segmentation"] = time.perf_counter() - _t0

        # 2. Нормализация типографики
        _t0 = time.perf_counter()
        text = self._run_plugins("typography", text, lang, is_before=True)
        normalizer = TypographyNormalizer(
            profile=self.options.profile,
            lang=lang,
        )
        text = normalizer.normalize(text)
        all_changes.extend(normalizer.changes)
        text = self._run_plugins("typography", text, lang, is_before=False)

        # 2b. Пользовательский словарь замен (custom_dict)
        if self.options.custom_dict:
            text, cd_changes = self._apply_custom_dict(text)
            all_changes.extend(cd_changes)
        stage_timings["typography"] = time.perf_counter() - _t0

        # 2c. CJK pre-segmentation — inject word boundaries for CJK text
        # so downstream word-level stages (regex \b, splits) work correctly.
        _cjk_active = False
        if is_cjk_text(text):
            _t0 = time.perf_counter()
            from texthumanize.cjk_segmenter import detect_cjk_lang
            cjk_lang = detect_cjk_lang(text) or "zh"
            _cjk_seg = CJKSegmenter(lang=cjk_lang)
            _cjk_words = _cjk_seg.segment(text)
            # Only add spaces between CJK tokens (keep existing if mixed)
            cjk_text = " ".join(w for w in _cjk_words if w.strip())
            if cjk_text and cjk_text != text:
                text = cjk_text
                _cjk_active = True
                all_changes.append({
                    "type": "cjk_segmentation",
                    "description": (
                        f"CJK сегментация ({cjk_lang}): "
                        f"разбивка на {len(_cjk_words)} токенов"
                    ),
                })
            stage_timings["cjk_segmentation"] = time.perf_counter() - _t0

        # Этапы 3-6: словарная обработка (Tier 1 + Tier 2 languages)
        # Skip if content is pure code — nothing to debureau/rephrase
        _skip_word_stages = (
            content_profile.content_type == ContentType.CODE
        )
        if (get_language_tier(lang) <= 2 and get_language_tier(lang) > 0
                and not _skip_word_stages):
            # 3. Деканцеляризация
            text = self._run_plugins("debureaucratization", text, lang, is_before=True)
            def _run_debureau() -> tuple[str, list]:
                d = Debureaucratizer(
                    lang=lang,
                    profile=effective_options.profile,
                    intensity=effective_options.intensity,
                    seed=effective_options.seed,
                )
                t = d.process(text)
                return t, d.changes
            text, _ch = self._safe_stage(
                "debureaucratization", text, lang,
                _run_debureau, stage_timings,
            )
            all_changes.extend(_ch)
            checkpoints.append(("debureaucratization", text))
            text = self._run_plugins("debureaucratization", text, lang, is_before=False)

            # 4. Разнообразие структуры
            text = self._run_plugins("structure", text, lang, is_before=True)
            def _run_structure() -> tuple[str, list]:
                s = StructureDiversifier(
                    lang=lang,
                    profile=effective_options.profile,
                    intensity=effective_options.intensity,
                    seed=effective_options.seed,
                )
                t = s.process(text)
                return t, s.changes
            text, _ch = self._safe_stage("structure", text, lang, _run_structure, stage_timings)
            all_changes.extend(_ch)
            checkpoints.append(("structure", text))
            text = self._run_plugins("structure", text, lang, is_before=False)

            # 5. Уменьшение повторов
            text = self._run_plugins("repetitions", text, lang, is_before=True)
            def _run_repetitions() -> tuple[str, list]:
                r = RepetitionReducer(
                    lang=lang,
                    profile=effective_options.profile,
                    intensity=effective_options.intensity,
                    seed=effective_options.seed,
                )
                t = r.process(text)
                return t, r.changes
            text, _ch = self._safe_stage("repetitions", text, lang, _run_repetitions, stage_timings)
            all_changes.extend(_ch)
            checkpoints.append(("repetitions", text))
            text = self._run_plugins("repetitions", text, lang, is_before=False)

            # 6. Инъекция «живости»
            text = self._run_plugins("liveliness", text, lang, is_before=True)
            def _run_liveliness() -> tuple[str, list]:
                li = LivelinessInjector(
                    lang=lang,
                    profile=effective_options.profile,
                    intensity=effective_options.intensity,
                    seed=effective_options.seed,
                )
                t = li.process(text)
                return t, li.changes
            text, _ch = self._safe_stage("liveliness", text, lang, _run_liveliness, stage_timings)
            all_changes.extend(_ch)
            checkpoints.append(("liveliness", text))
            text = self._run_plugins("liveliness", text, lang, is_before=False)

        # 7. Семантическое перефразирование (Tier 1 + Tier 2)
        #    Skipped if content type disallows paraphrasing (e.g. pure code)
        if (get_language_tier(lang) <= 2 and get_language_tier(lang) > 0
                and content_profile.allow_paraphrase):
            text = self._run_plugins("paraphrasing", text, lang, is_before=True)
            def _run_paraphrasing() -> tuple[str, list]:
                p = SemanticParaphraser(
                    lang=lang,
                    intensity=effective_options.intensity / 100.0,
                    seed=effective_options.seed,
                )
                t = p.process(text)
                changes = [
                    {"type": "paraphrasing", "kind": c.kind,
                     "description": f"{c.kind}: {c.original[:60]}… → {c.transformed[:60]}…"}
                    for c in p.changes
                ]
                return t, changes
            text, _ch = self._safe_stage(
                "paraphrasing", text, lang,
                _run_paraphrasing, stage_timings,
            )
            all_changes.extend(_ch)
            checkpoints.append(("paraphrasing", text))
            text = self._run_plugins("paraphrasing", text, lang, is_before=False)

        # 7b. Syntax rewriting (sentence-level structural transforms)
        # Only for Tier 1 languages — SyntaxRewriter has proper grammar
        # rules only for these. Skipped for code/academic/technical content.
        if get_language_tier(lang) <= 2 and get_language_tier(lang) > 0 and content_profile.allow_syntax_rewrite:
            text = self._run_plugins("syntax_rewriting", text, lang, is_before=True)
            def _run_syntax_rewrite() -> tuple[str, list]:
                import re as _re_sr
                sr = SyntaxRewriter(
                    lang=lang,
                    seed=effective_options.seed,
                )
                # Split into paragraphs first to preserve structure
                paragraphs = text.split('\n')
                changed = False
                prob = effective_options.intensity / 100.0
                import random as _rnd_sr
                _sr_rng = _rnd_sr.Random(effective_options.seed)
                result_paras = []
                for para in paragraphs:
                    if not para.strip():
                        result_paras.append(para)
                        continue
                    sents = _re_sr.split(r'(?<=[.!?])\s+', para)
                    rewritten = []
                    for s in sents:
                        if _sr_rng.random() < prob * 0.55:
                            r = sr.rewrite_random(s)
                            if r != s:
                                changed = True
                            rewritten.append(r)
                        else:
                            rewritten.append(s)
                    result_paras.append(' '.join(rewritten))
                t = '\n'.join(result_paras)
                changes = [{"type": "syntax_rewrite", "description": "Syntax rewriting applied"}]
                return (t, changes) if changed else (text, [])
            text, _ch = self._safe_stage(
                "syntax_rewriting", text, lang,
                _run_syntax_rewrite, stage_timings,
            )
            all_changes.extend(_ch)
            checkpoints.append(("syntax_rewriting", text))
            # Sentence-level validation: revert broken sentences
            _sv_before = checkpoints[-2][1] if len(checkpoints) >= 2 else original
            text = _sv.validate(_sv_before, text, stage_name="syntax_rewriting")
            text = self._run_plugins("syntax_rewriting", text, lang, is_before=False)

        # 8. Гармонизация тона (для ВСЕХ языков)
        text = self._run_plugins("tone", text, lang, is_before=True)
        def _run_tone() -> tuple[str, list]:
            th = ToneHarmonizer(
                lang=lang,
                profile=effective_options.profile,
                intensity=effective_options.intensity,
                seed=effective_options.seed,
            )
            t = th.process(text)
            return t, th.changes
        text, _ch = self._safe_stage("tone", text, lang, _run_tone, stage_timings)
        all_changes.extend(_ch)
        checkpoints.append(("tone", text))
        text = self._run_plugins("tone", text, lang, is_before=False)

        # 9. Универсальная обработка (для ВСЕХ языков)
        text = self._run_plugins("universal", text, lang, is_before=True)
        def _run_universal() -> tuple[str, list]:
            u = UniversalProcessor(
                profile=effective_options.profile,
                intensity=effective_options.intensity,
                seed=effective_options.seed,
            )
            t = u.process(text)
            return t, u.changes
        text, _ch = self._safe_stage("universal", text, lang, _run_universal, stage_timings)
        all_changes.extend(_ch)
        checkpoints.append(("universal", text))
        text = self._run_plugins("universal", text, lang, is_before=False)

        # 10. Натурализация стиля (КЛЮЧЕВОЙ ЭТАП — для ВСЕХ языков)
        text = self._run_plugins("naturalization", text, lang, is_before=True)
        def _run_naturalization() -> tuple[str, list]:
            n = TextNaturalizer(
                lang=lang,
                profile=effective_options.profile,
                intensity=effective_options.intensity,
                seed=effective_options.seed,
            )
            t = n.process(text)
            return t, n.changes
        text, _ch = self._safe_stage(
            "naturalization", text, lang,
            _run_naturalization, stage_timings,
        )
        all_changes.extend(_ch)
        checkpoints.append(("naturalization", text))
        # Sentence-level validation: revert broken sentences
        _sv_before = checkpoints[-2][1] if len(checkpoints) >= 2 else original
        text = _sv.validate(_sv_before, text, stage_name="naturalization")
        text = self._run_plugins("naturalization", text, lang, is_before=False)

        # 10a. Paraphrase engine — structural rewrites (MWE simplification,
        # connector deletion, hedging, perspective rotation, clause embedding).
        text = self._run_plugins("paraphrase_engine", text, lang, is_before=True)
        def _run_paraphrase_engine() -> tuple[str, list]:
            from texthumanize.paraphrase_engine import ParaphraseEngine
            pe = ParaphraseEngine(
                lang=lang,
                intensity=effective_options.intensity,
                seed=effective_options.seed,
            )
            t = pe.transform(text)
            return t, [{"type": "paraphrase_engine", "description": c} for c in pe.changes]
        text, _ch = self._safe_stage(
            "paraphrase_engine", text, lang,
            _run_paraphrase_engine, stage_timings,
        )
        all_changes.extend(_ch)
        checkpoints.append(("paraphrase_engine", text))
        # Sentence-level validation: revert broken sentences
        _sv_before = checkpoints[-2][1] if len(checkpoints) >= 2 else original
        text = _sv.validate(_sv_before, text, stage_name="paraphrase_engine")
        text = self._run_plugins("paraphrase_engine", text, lang, is_before=False)

        # 10a½. Sentence restructuring — deep structural transforms
        # (contractions, register mixing, length reshaping, discourse markers,
        #  rhetorical questions, cleft/existential transforms).
        text = self._run_plugins("sentence_restructuring", text, lang, is_before=True)
        def _run_restructuring() -> tuple[str, list]:
            from texthumanize.sentence_restructurer import SentenceRestructurer
            sr = SentenceRestructurer(
                lang=lang,
                intensity=effective_options.intensity,
                seed=effective_options.seed,
            )
            t = sr.process(text)
            return t, sr.changes
        text, _ch = self._safe_stage(
            "sentence_restructuring", text, lang,
            _run_restructuring, stage_timings,
        )
        all_changes.extend(_ch)
        checkpoints.append(("sentence_restructuring", text))
        # Sentence-level validation: revert broken sentences
        _sv_before = checkpoints[-2][1] if len(checkpoints) >= 2 else original
        text = _sv.validate(_sv_before, text, stage_name="sentence_restructuring")
        text = self._run_plugins("sentence_restructuring", text, lang, is_before=False)

        # 10b. Word LM quality gate — advisory perplexity monitoring.
        # Reports perplexity change but does NOT roll back — perplexity
        # values are advisory until language model data is expanded.
        _t0 = time.perf_counter()
        try:
            _wlm = WordLanguageModel(lang=lang)
            _pp_before = _wlm.perplexity(checkpoints[-2][1] if len(checkpoints) >= 2 else original)
            _pp_after = _wlm.perplexity(text)
            if _pp_before > 0 and _pp_after > 0:
                _pp_delta = "↓" if _pp_after < _pp_before else "↑"
                all_changes.append({
                    "type": "quality_gate_advisory",
                    "description": (
                        f"Word LM: перплексия {_pp_before:.1f}→{_pp_after:.1f} ({_pp_delta})"
                    ),
                })
        except Exception:
            pass  # Word LM is advisory, never blocks pipeline
        stage_timings["word_lm_gate"] = time.perf_counter() - _t0

        # 10c. Entropy & burstiness injection (Phase 1 — all languages)
        text = self._run_plugins("entropy_injection", text, lang, is_before=True)
        def _run_entropy() -> tuple[str, list]:
            from texthumanize.entropy_injector import EntropyInjector
            ei = EntropyInjector(
                lang=lang,
                intensity=effective_options.intensity,
                seed=effective_options.seed,
                profile=effective_options.profile,
            )
            t = ei.process(text)
            return t, ei.changes
        text, _ch = self._safe_stage(
            "entropy_injection", text, lang,
            _run_entropy, stage_timings,
        )
        all_changes.extend(_ch)
        checkpoints.append(("entropy_injection", text))
        # Sentence-level validation: revert broken sentences
        _sv_before = checkpoints[-2][1] if len(checkpoints) >= 2 else original
        text = _sv.validate(_sv_before, text, stage_name="entropy_injection")
        text = self._run_plugins("entropy_injection", text, lang, is_before=False)

        # 10d. Post-injection fragment deduplication
        # Multiple injection stages (burstiness, rhetorical questions,
        # discourse surprise) fire independently, so adjacent short
        # fragments can stack into garbled chains like
        # "Here's the deal: but why? Granted, and, but..."
        # Strip excess: keep at most 1 fragment per 5-sentence window.
        text = self._strip_fragment_chains(text, lang)

        # 11. Оптимизация читаемости (для ВСЕХ языков)
        text = self._run_plugins("readability", text, lang, is_before=True)
        def _run_readability() -> tuple[str, list]:
            ro = ReadabilityOptimizer(
                lang=lang,
                profile=effective_options.profile,
                intensity=effective_options.intensity,
                seed=effective_options.seed,
            )
            t = ro.process(text)
            return t, ro.changes
        text, _ch = self._safe_stage("readability", text, lang, _run_readability, stage_timings)
        all_changes.extend(_ch)
        checkpoints.append(("readability", text))
        text = self._run_plugins("readability", text, lang, is_before=False)

        # 12. Грамматическая коррекция (для ВСЕХ языков — финальная полировка)
        text = self._run_plugins("grammar", text, lang, is_before=True)
        def _run_grammar() -> tuple[str, list]:
            g = GrammarCorrector(
                lang=lang,
                profile=effective_options.profile,
                intensity=effective_options.intensity,
                seed=effective_options.seed,
            )
            t = g.process(text)
            return t, g.changes
        text, _ch = self._safe_stage("grammar", text, lang, _run_grammar, stage_timings)
        all_changes.extend(_ch)
        checkpoints.append(("grammar", text))
        text = self._run_plugins("grammar", text, lang, is_before=False)

        # 12½. Grammar Guard — neural artifact detector + synonym rollback.
        # Catches bad collocations/agreements introduced by upstream stages.
        text = self._run_plugins("grammar_guard", text, lang, is_before=True)
        def _run_grammar_guard() -> tuple[str, list]:
            from texthumanize.grammar_guard import GrammarGuard
            gg = GrammarGuard(lang=lang)
            t = gg.process(text, original)
            return t, gg.result.changes
        text, _ch = self._safe_stage(
            "grammar_guard", text, lang,
            _run_grammar_guard, stage_timings,
        )
        all_changes.extend(_ch)
        checkpoints.append(("grammar_guard", text))
        # Sentence-level validation after grammar guard
        _sv_before = checkpoints[-2][1] if len(checkpoints) >= 2 else original
        text = _sv.validate(_sv_before, text, stage_name="grammar_guard")
        text = self._run_plugins("grammar_guard", text, lang, is_before=False)

        # 13. Коррекция когерентности (для ВСЕХ языков)
        text = self._run_plugins("coherence", text, lang, is_before=True)
        def _run_coherence() -> tuple[str, list]:
            c = CoherenceRepairer(
                lang=lang,
                profile=effective_options.profile,
                intensity=effective_options.intensity,
                seed=effective_options.seed,
            )
            t = c.process(text)
            return t, c.changes
        text, _ch = self._safe_stage("coherence", text, lang, _run_coherence, stage_timings)
        all_changes.extend(_ch)
        checkpoints.append(("coherence", text))
        # Sentence-level validation after coherence
        _sv_before = checkpoints[-2][1] if len(checkpoints) >= 2 else original
        text = _sv.validate(_sv_before, text, stage_name="coherence")
        text = self._run_plugins("coherence", text, lang, is_before=False)

        # 13a. Final entropy/burstiness re-injection
        # Stages 11-13 (readability, grammar, coherence) can re-uniformize
        # sentence lengths and undo entropy work. This final pass ensures
        # burstiness and statistical diversity are maintained.
        if effective_options.intensity >= 20:
            def _run_entropy_final() -> tuple[str, list]:
                from texthumanize.entropy_injector import EntropyInjector
                ei = EntropyInjector(
                    lang=lang,
                    intensity=min(effective_options.intensity, 50),
                    seed=(effective_options.seed + 7) if effective_options.seed else None,
                    profile=effective_options.profile,
                )
                t = ei.process(text)
                return t, [c for c in ei.changes if "burstiness" in str(c.get("type", ""))]
            text, _ch = self._safe_stage(
                "entropy_final", text, lang,
                _run_entropy_final, stage_timings,
            )
            all_changes.extend(_ch)

        # 13a¼. Final sentence-level cleanup
        # Late stages (readability, grammar, coherence, entropy_final) can
        # re-introduce artifacts that _strip_fragment_chains already cleaned.
        # Apply targeted fixes:
        import re as _re_late
        #  (a) Double conjunctions: "and and", "и и", "but but"
        text = _re_late.sub(
            r'\b(and|but|or|yet|so|и|і|а|але|но|und|oder|aber|et|ou|mais|y|o|pero)\s+\1\b',
            r'\1', text, flags=_re_late.IGNORECASE,
        )
        #  (b) Garbled conjunction chains left over: ", and, but, however, and"
        text = _re_late.sub(
            r'(?:,\s*(?:and|but|or|however|moreover|also|and)\s*){2,}',
            ', ', text, flags=_re_late.IGNORECASE,
        )

        # 13a½. Sentence-level validation summary
        if _sv.total_reverts > 0:
            all_changes.append({
                "type": "sentence_validation",
                "description": (
                    f"Валидация предложений: откачено {_sv.total_reverts} "
                    f"сломанных предложений"
                ),
            })

        # 13b. Anti-fingerprint diversification
        _fp_rand = FingerprintRandomizer(
            seed=effective_options.seed,
            jitter_level=effective_options.intensity / 100.0 * 0.5,
        )
        text = _fp_rand.diversify_output(text)

        # 14. Восстановление защищённых сегментов
        _t0 = time.perf_counter()
        text = segmented.restore(text)
        stage_timings["restore"] = time.perf_counter() - _t0

        # ── Safety: collapse overlapping em-dash asides ──
        import re as _re_dash
        text = _re_dash.sub(r'\u2014\s*\u2014', '\u2014', text)

        # 15. Валидация
        _t0 = time.perf_counter()
        _user_max_v = self.options.constraints.get("max_change_ratio", None)
        if _user_max_v is not None:
            max_change_v = _user_max_v
        else:
            _iv = effective_options.intensity / 100.0
            max_change_v = min(0.70, 0.25 + _iv * 0.40)
        validator = QualityValidator(
            lang=lang,
            max_change_ratio=max_change_v,
            keep_keywords=keep_kw,
        )
        validation = validator.validate(original, text, metrics_before)

        # Откат только при критических ошибках (потеря ключевых слов,
        # резкий рост AI-скора). Слишком высокий change_ratio не вызывает
        # откат — graduated retry обработает это.
        critical_errors = [e for e in validation.errors if "ключевое слово" in e.lower()
                          or "искусственность" in e.lower()]
        if critical_errors:
            # Try partial rollback — remove stages from the end
            for cp_name, cp_text in reversed(checkpoints):
                restored_cp = segmented.restore(cp_text)
                cp_valid = validator.validate(original, restored_cp, metrics_before)
                cp_critical = [e for e in cp_valid.errors if "ключевое слово" in e.lower()
                              or "искусственность" in e.lower()]
                if not cp_critical:
                    text = restored_cp
                    all_changes.append({
                        "type": "partial_rollback",
                        "description": f"Частичный откат до этапа «{cp_name}»",
                    })
                    break
            else:
                # Full rollback if no checkpoint is clean
                text = original
                all_changes = [{
                    "type": "rollback",
                    "description": f"Полный откат: {'; '.join(critical_errors)}",
                }]
        stage_timings["validation"] = time.perf_counter() - _t0

        # Анализ после обработки
        metrics_after = analyzer.analyze(text)

        # Стилистический анализ после обработки (если задан target)
        if target_fp is not None and isinstance(target_fp, StylisticFingerprint):
            style_analyzer_post = StylisticAnalyzer(lang=lang)
            result_fp = style_analyzer_post.extract(text)
            style_meta["style_similarity_after"] = round(
                result_fp.similarity(target_fp), 3,
            )

        return HumanizeResult(
            original=original,
            text=text,
            lang=lang,
            profile=self.options.profile,
            intensity=self.options.intensity,
            changes=all_changes,
            metrics_before={
                "artificiality_score": metrics_before.artificiality_score,
                "avg_sentence_length": metrics_before.avg_sentence_length,
                "bureaucratic_ratio": metrics_before.bureaucratic_ratio,
                "connector_ratio": metrics_before.connector_ratio,
                "repetition_score": metrics_before.repetition_score,
                "typography_score": metrics_before.typography_score,
                "predictability_score": metrics_before.predictability_score,
                "vocabulary_richness": metrics_before.vocabulary_richness,
                "content_type": content_profile.content_type.value,
                "content_confidence": round(content_profile.confidence, 3),
                **style_meta,
            },
            metrics_after={
                "artificiality_score": metrics_after.artificiality_score,
                "avg_sentence_length": metrics_after.avg_sentence_length,
                "bureaucratic_ratio": metrics_after.bureaucratic_ratio,
                "connector_ratio": metrics_after.connector_ratio,
                "repetition_score": metrics_after.repetition_score,
                "typography_score": metrics_after.typography_score,
                "predictability_score": metrics_after.predictability_score,
                "vocabulary_richness": metrics_after.vocabulary_richness,
                "stage_timings": stage_timings,
                "total_time": sum(stage_timings.values()),
            },
        )



"""Tests for OpenAI LLM integration in the humanization pipeline."""
from __future__ import annotations

import unittest
from unittest.mock import MagicMock, patch

from texthumanize import humanize
from texthumanize.pipeline import HumanizeOptions, Pipeline


class TestLLMOptionsPropagation(unittest.TestCase):
    """openai_api_key and openai_model flow through correctly."""

    def test_options_fields_exist(self):
        """HumanizeOptions has openai_api_key and openai_model fields."""
        opts = HumanizeOptions()
        assert opts.openai_api_key is None
        assert opts.openai_model == "gpt-4o-mini"

    def test_options_custom_values(self):
        opts = HumanizeOptions(openai_api_key="sk-test123", openai_model="gpt-4o")
        assert opts.openai_api_key == "sk-test123"
        assert opts.openai_model == "gpt-4o"

    def test_humanize_accepts_openai_params(self):
        """humanize() accepts openai_api_key/openai_model without error."""
        # With a fake key the LLM step should fail gracefully and
        # still return a locally-humanized result.
        result = humanize(
            "This is a test sentence for humanization.",
            lang="en",
            openai_api_key="sk-fake-key-for-testing",
            openai_model="gpt-4o-mini",
            seed=42,
        )
        assert result.text
        assert isinstance(result.text, str)

    def test_no_openai_key_skips_llm(self):
        """Without an API key, LLM-assisted evasion is skipped."""
        result = humanize(
            "Furthermore, the implementation of this technology.",
            lang="en",
            seed=42,
        )
        change_types = [c["type"] for c in result.changes]
        assert "llm_evasion" not in change_types


class TestLLMAssistedRewrite(unittest.TestCase):
    """Unit tests for Pipeline._llm_assisted_rewrite()."""

    def _make_pipeline(self, api_key: str = "sk-test") -> Pipeline:
        opts = HumanizeOptions(
            lang="en",
            openai_api_key=api_key,
            openai_model="gpt-4o-mini",
            seed=42,
        )
        return Pipeline(opts)

    def _make_result(self, text: str, original: str = "original text") -> MagicMock:
        result = MagicMock()
        result.text = text
        result.changes = []
        result.metrics_before = {}
        result.metrics_after = {}
        return result

    @patch("texthumanize.ai_backend.AIBackend")
    def test_llm_rewrite_improves_score(self, mock_backend_cls):
        """When LLM produces a better score, it's returned."""
        mock_ai = MagicMock()
        mock_ai.paraphrase.return_value = "A totally human-sounding rewrite."
        mock_ai.improve_naturalness.return_value = "Another natural version."
        mock_backend_cls.return_value = mock_ai

        pipeline = self._make_pipeline()
        current = self._make_result("AI-sounding text with formal structure.")

        # Mock detect_fn: original text=0.8, both candidates=0.3
        def mock_detect(text, *, lang):
            if text == "AI-sounding text with formal structure.":
                return {"combined_score": 0.8}
            return {"combined_score": 0.3}

        result = pipeline._llm_assisted_rewrite(
            original="Some original text.",
            current=current,
            lang="en",
            api_key="sk-test",
            model="gpt-4o-mini",
            target_score=0.40,
            max_change=0.60,
            detect_fn=mock_detect,
            check_deadline=lambda: None,
        )

        assert result is not None
        assert result.text in (
            "A totally human-sounding rewrite.",
            "Another natural version.",
        )
        change_types = [c["type"] for c in result.changes]
        assert "llm_evasion" in change_types

    @patch("texthumanize.ai_backend.AIBackend")
    def test_llm_rewrite_no_improvement(self, mock_backend_cls):
        """When LLM doesn't improve score, None is returned."""
        mock_ai = MagicMock()
        mock_ai.paraphrase.return_value = "Still AI text."
        mock_ai.improve_naturalness.return_value = "Still AI too."
        mock_backend_cls.return_value = mock_ai

        pipeline = self._make_pipeline()
        current = self._make_result("Already at score 0.9.")

        def mock_detect(text, *, lang):
            return {"combined_score": 0.9}  # Always bad

        result = pipeline._llm_assisted_rewrite(
            original="Some original.",
            current=current,
            lang="en",
            api_key="sk-test",
            model="gpt-4o-mini",
            target_score=0.40,
            max_change=0.60,
            detect_fn=mock_detect,
            check_deadline=lambda: None,
        )

        assert result is None

    @patch("texthumanize.ai_backend.AIBackend")
    def test_llm_rewrite_backend_error(self, mock_backend_cls):
        """Backend errors return None gracefully."""
        mock_backend_cls.side_effect = Exception("Connection error")

        pipeline = self._make_pipeline()
        current = self._make_result("Some text.")

        result = pipeline._llm_assisted_rewrite(
            original="Original.",
            current=current,
            lang="en",
            api_key="sk-test",
            model="gpt-4o-mini",
            target_score=0.40,
            max_change=0.60,
            detect_fn=lambda t, *, lang: {"combined_score": 0.8},
            check_deadline=lambda: None,
        )

        assert result is None

    @patch("texthumanize.ai_backend.AIBackend")
    def test_llm_paraphrase_empty_response(self, mock_backend_cls):
        """Empty paraphrase response returns None."""
        mock_ai = MagicMock()
        mock_ai.paraphrase.return_value = ""
        mock_backend_cls.return_value = mock_ai

        pipeline = self._make_pipeline()
        current = self._make_result("Some text.")

        result = pipeline._llm_assisted_rewrite(
            original="Original.",
            current=current,
            lang="en",
            api_key="sk-test",
            model="gpt-4o-mini",
            target_score=0.40,
            max_change=0.60,
            detect_fn=lambda t, *, lang: {"combined_score": 0.8},
            check_deadline=lambda: None,
        )

        assert result is None


class TestLLMInRunMethod(unittest.TestCase):
    """Integration: LLM block in Pipeline.run()."""

    @patch("texthumanize.ai_backend.AIBackend")
    def test_llm_block_called_when_key_present_and_score_high(self, mock_cls):
        """LLM block is attempted when API key is set and score > target."""
        mock_ai = MagicMock()
        # Return something vaguely human
        mock_ai.paraphrase.return_value = (
            "AI stuff got better with simpler words and shorter bits."
        )
        mock_ai.improve_naturalness.return_value = None
        mock_cls.return_value = mock_ai

        # Use a clearly AI text so the detection loop doesn't already
        # bring it below threshold before the LLM block.
        text = (
            "Furthermore, the utilization of this methodology is paramount."
        )
        result = humanize(
            text,
            lang="en",
            openai_api_key="sk-fake",
            seed=42,
            intensity=30,  # low intensity to keep local transforms mild
        )
        # Result should exist regardless
        assert result.text
        assert isinstance(result.text, str)


if __name__ == "__main__":
    unittest.main()



"""Тесты для модуля health_score — комплексная оценка качества контента."""

from __future__ import annotations

import pytest

from texthumanize.health_score import (
    ContentHealthReport,
    HealthComponent,
    content_health,
)


class TestContentHealth:
    """Tests for content_health()."""

    def test_returns_report(self):
        text = (
            "The quick brown fox jumps over the lazy dog. "
            "Mountains rise in the distance. Rivers flow gently."
        )
        report = content_health(text, lang="en")
        assert isinstance(report, ContentHealthReport)
        assert 0 <= report.score <= 100
        assert report.grade in ("A+", "A", "B", "C", "D", "F")

    def test_components_present(self):
        """Report should have multiple health components."""
        report = content_health(
            "This is a test sentence. Here is another one. And a third.",
            lang="en",
        )
        assert isinstance(report.components, list)
        assert len(report.components) >= 1
        for comp in report.components:
            assert isinstance(comp, HealthComponent)
            assert hasattr(comp, "name")
            assert hasattr(comp, "score")
            assert hasattr(comp, "weight")

    def test_grade_mapping(self):
        """Grade should be derived from score."""
        report = content_health("Normal text with reasonable quality.", lang="en")
        if report.score >= 90:
            assert report.grade in ("A+", "A")
        elif report.score >= 80:
            assert report.grade in ("A", "B")

    def test_include_flags(self):
        """Disabling components should still work."""
        report = content_health(
            "Test text for health.",
            lang="en",
            include_ai=False,
            include_grammar=False,
            include_uniqueness=False,
        )
        assert isinstance(report, ContentHealthReport)

    def test_empty_text(self):
        """Empty text should not crash."""
        report = content_health("", lang="en")
        assert isinstance(report, ContentHealthReport)

    @pytest.mark.parametrize("lang", ["en", "ru", "de", "fr", "es"])
    def test_multiple_languages(self, lang: str):
        """Should work for multiple languages."""
        report = content_health("Some test text here. Another sentence.", lang=lang)
        assert isinstance(report, ContentHealthReport)
        assert 0 <= report.score <= 100

    def test_high_quality_text(self):
        """Well-written text should score reasonably."""
        text = (
            "Natural language processing has evolved significantly. "
            "Modern approaches combine statistical methods with deep learning. "
            "Researchers continue exploring novel architectures. "
            "Applications range from translation to summarization."
        )
        report = content_health(text, lang="en")
        assert report.score >= 30  # At least not terrible

    def test_component_weights_sum(self):
        """Component weights should be positive."""
        report = content_health("Test sentence here.", lang="en")
        for comp in report.components:
            assert comp.weight > 0



"""Тесты для coherence, paraphrase, watermark — повышение покрытия."""
from __future__ import annotations

from texthumanize.coherence import CoherenceAnalyzer
from texthumanize.paraphrase import Paraphraser
from texthumanize.watermark import WatermarkDetector, detect_watermarks

# ═══════════════════════════════════════════════════════════════
#  COHERENCE — с 68% до ~85%+
# ═══════════════════════════════════════════════════════════════

class TestCoherenceAnalyzer:
    """Тесты анализатора когерентности."""

    def setup_method(self):
        self.ca = CoherenceAnalyzer(lang="en")
        self.ca_ru = CoherenceAnalyzer(lang="ru")

    def test_single_paragraph(self):
        """Один абзац — нет причин для низкой когерентности."""
        report = self.ca.analyze("This is a single paragraph with several sentences. "
                                 "It discusses one topic. Nothing else here.")
        assert report.overall > 0
        assert report.paragraph_count == 1

    def test_multiple_paragraphs(self):
        """Несколько абзацев — полный анализ."""
        text = (
            "First paragraph about technology. Computers are everywhere.\n\n"
            "However, the second paragraph talks about related topics. "
            "Technology impacts our daily lives significantly.\n\n"
            "Furthermore, the third paragraph provides a conclusion. "
            "We must adapt to technological changes."
        )
        report = self.ca.analyze(text)
        assert report.paragraph_count == 3
        assert 0 <= report.overall <= 1
        assert 0 <= report.lexical_cohesion <= 1
        assert 0 <= report.transition_score <= 1
        assert 0 <= report.topic_consistency <= 1
        assert 0 <= report.sentence_opening_diversity <= 1

    def test_uniform_paragraph_lengths(self):
        """Одинаковые длины абзацев → issue."""
        text = (
            "First short paragraph here.\n\n"
            "Second short paragraph here.\n\n"
            "Third short paragraph here.\n\n"
            "Fourth short paragraph here."
        )
        report = self.ca.analyze(text)
        # Может или может не иметь issue в зависимости от CV
        assert isinstance(report.issues, list)

    def test_very_long_paragraphs(self):
        """Очень длинный абзац → issue."""
        long_para = " ".join(["word"] * 250) + "."
        text = f"Short intro.\n\n{long_para}\n\nShort conclusion."
        report = self.ca.analyze(text)
        assert "very_long_paragraphs" in report.issues

    def test_very_short_paragraphs(self):
        """Очень короткий абзац → issue."""
        text = "Full intro paragraph with details.\n\nYes.\n\nFull conclusion paragraph."
        report = self.ca.analyze(text)
        assert "very_short_paragraphs" in report.issues

    def test_suggest_improvements_uniform(self):
        """Рекомендации для однородных абзацев."""
        text = (
            "First short paragraph here.\n\n"
            "Second short paragraph here.\n\n"
            "Third short paragraph here.\n\n"
            "Fourth short paragraph here."
        )
        report = self.ca.analyze(text)
        report.issues.append("paragraph_uniform_length")
        suggestions = self.ca.suggest_improvements(text, report)
        assert any("uniform" in s.lower() or "cv" in s.lower() for s in suggestions)

    def test_suggest_improvements_few_transitions(self):
        """Рекомендации при отсутствии переходов."""
        text = (
            "Apples are red. They taste good.\n\n"
            "Bananas are yellow. Eating them is fun.\n\n"
            "Grapes are purple. Wine comes from them."
        )
        report = self.ca.analyze(text)
        report.issues.append("few_transitions")
        suggestions = self.ca.suggest_improvements(text, report)
        assert any("transition" in s.lower() for s in suggestions)

    def test_suggest_improvements_low_cohesion(self):
        """Рекомендации при низкой лексической связности."""
        text = "Technology is great.\n\nCats are fluffy.\n\nPizzas are delicious."
        report = self.ca.analyze(text)
        report.issues.append("low_lexical_cohesion")
        suggestions = self.ca.suggest_improvements(text, report)
        assert any("cohesion" in s.lower() for s in suggestions)

    def test_suggest_improvements_repetitive_openings(self):
        """Рекомендации при повторяющихся началах."""
        text = "Short text.\n\nAnother short text."
        report = self.ca.analyze(text)
        report.issues.append("repetitive_sentence_openings")
        suggestions = self.ca.suggest_improvements(text, report)
        assert any("opening" in s.lower() or "vary" in s.lower() for s in suggestions)

    def test_suggest_improvements_long_paragraphs(self):
        """Рекомендации при очень длинных абзацах."""
        text = "Short.\n\n" + " ".join(["word"] * 250) + "."
        report = self.ca.analyze(text)
        report.issues.append("very_long_paragraphs")
        suggestions = self.ca.suggest_improvements(text, report)
        assert any("200" in s or "break" in s.lower() for s in suggestions)

    def test_suggest_auto_analyze(self):
        """suggest_improvements без предварительного report."""
        text = (
            "First topic discussed here.\n\n"
            "Second topic is different.\n\n"
            "Third topic wraps up."
        )
        suggestions = self.ca.suggest_improvements(text)
        assert isinstance(suggestions, list)

    def test_cosine_similarity_empty(self):
        """Косинусное сходство пустых Counter."""
        from collections import Counter
        assert CoherenceAnalyzer._cosine_similarity(Counter(), Counter()) == 0.0
        assert CoherenceAnalyzer._cosine_similarity(Counter(a=1), Counter()) == 0.0

    def test_topic_consistency_few_paragraphs(self):
        """Тематическая последовательность при < 3 абзацах."""
        text = "First paragraph.\n\nSecond paragraph."
        report = self.ca.analyze(text)
        # Должна вернуть 0.8 для 2 абзацев
        assert report.topic_consistency >= 0.5

    def test_opening_diversity_few_sentences(self):
        """Разнообразие открытий при < 3 предложениях."""
        text = "One sentence."
        report = self.ca.analyze(text)
        # С 1 предложением diversity = 0.0 (нечего сравнивать)
        assert report.sentence_opening_diversity >= 0.0

    def test_overall_computation(self):
        """Итоговый балл всегда в [0, 1]."""
        text = (
            "Introduction to the topic. Background information.\n\n"
            "However, the main point is different. Details follow.\n\n"
            "In conclusion, results are clear."
        )
        report = self.ca.analyze(text)
        assert 0.0 <= report.overall <= 1.0


# ═══════════════════════════════════════════════════════════════
#  PARAPHRASE — с 71% до ~85%+
# ═══════════════════════════════════════════════════════════════

class TestParaphraser:
    """Тесты парафразера."""

    def test_short_text(self):
        """Короткий текст возвращается без изменений."""
        p = Paraphraser(lang="en", seed=42, intensity=0.5)
        result = p.paraphrase("Hello world.")
        assert isinstance(result.paraphrased, str)
        assert len(result.paraphrased) > 0

    def test_clause_swap(self):
        """Перестановка клауз (Although X, Y → Y, although X)."""
        p = Paraphraser(lang="en", seed=42, intensity=1.0)
        text = "Although the system works well, there are issues."
        result = p.paraphrase(text)
        assert isinstance(result.paraphrased, str)

    def test_passive_to_active(self):
        """Пассив → актив."""
        p = Paraphraser(lang="en", seed=42, intensity=1.0)
        text = "The report was written by the team."
        result = p.paraphrase(text)
        assert isinstance(result.paraphrased, str)

    def test_passive_non_english(self):
        """Пассив не работает для других языков."""
        p = Paraphraser(lang="ru", seed=42, intensity=1.0)
        text = "Отчёт был написан командой."
        result = p.paraphrase(text)
        assert isinstance(result.paraphrased, str)

    def test_sentence_split_long(self):
        """Разбиение длинного предложения."""
        p = Paraphraser(lang="en", seed=42, intensity=1.0)
        text = ("The system provides important data for analysis and "
                "the team uses this data to make critical decisions about "
                "the future direction of the project and the company strategy.")
        result = p.paraphrase(text)
        assert isinstance(result.paraphrased, str)

    def test_adverb_fronting(self):
        """Вынос наречия в начало предложения."""
        p = Paraphraser(lang="en", seed=42, intensity=1.0)
        text = "She completed the project successfully."
        result = p.paraphrase(text)
        assert isinstance(result.paraphrased, str)

    def test_adverb_fronting_ru(self):
        """Вынос наречия для русского."""
        p = Paraphraser(lang="ru", seed=42, intensity=1.0)
        text = "Она завершила проект успешно."
        result = p.paraphrase(text)
        assert isinstance(result.paraphrased, str)

    def test_nominalization(self):
        """Номинализация: глагол → существительное."""
        p = Paraphraser(lang="en", seed=42, intensity=1.0)
        text = "They decided to implement the new system."
        result = p.paraphrase(text)
        assert isinstance(result.paraphrased, str)

    def test_multiple_sentences(self):
        """Несколько предложений."""
        p = Paraphraser(lang="en", seed=42, intensity=0.8)
        text = (
            "Although data is important, many companies ignore it. "
            "The system processes information quickly. "
            "She completed the task efficiently."
        )
        result = p.paraphrase(text)
        assert isinstance(result.paraphrased, str)
        assert len(result.paraphrased) > 0

    def test_zero_intensity(self):
        """Интенсивность 0 — минимальные изменения."""
        p = Paraphraser(lang="en", seed=42, intensity=0.0)
        text = "The quick brown fox jumps over the lazy dog."
        result = p.paraphrase(text)
        assert isinstance(result.paraphrased, str)

    def test_high_intensity(self):
        """Максимальная интенсивность."""
        p = Paraphraser(lang="en", seed=42, intensity=1.0)
        text = (
            "Although the project was complex, it succeeded. "
            "The report was reviewed by all members. "
            "They analyzed the results carefully."
        )
        result = p.paraphrase(text)
        assert isinstance(result.paraphrased, str)

    def test_paraphrase_sentence_method(self):
        """Перефразирование одного предложения."""
        p = Paraphraser(lang="en", seed=42, intensity=1.0)
        transformed, change_type = p.paraphrase_sentence("Although X is true, Y follows.")
        assert isinstance(transformed, str)

    def test_result_has_changes(self):
        """Результат содержит список изменений."""
        p = Paraphraser(lang="en", seed=42, intensity=1.0)
        result = p.paraphrase(
            "Although the project was complex, it succeeded. "
            "She completed the task efficiently."
        )
        assert isinstance(result.changes, list)
        assert isinstance(result.confidence, float)


# ═══════════════════════════════════════════════════════════════
#  WATERMARK — с 74% до ~85%+
# ═══════════════════════════════════════════════════════════════

class TestWatermarkDetector:
    """Тесты детектора водяных знаков."""

    def setup_method(self):
        self.wd = WatermarkDetector(lang="en")

    def test_clean_text(self):
        """Чистый текст — нет водяных знаков."""
        report = self.wd.detect("This is a clean text without any watermarks.")
        assert not report.has_watermarks
        assert report.characters_removed == 0

    def test_zero_width_chars(self):
        """Zero-width символы обнаруживаются и удаляются."""
        text = "Hello\u200b world\u200c test\u200d here\ufeff end"
        report = self.wd.detect(text)
        assert report.has_watermarks
        assert any("zero_width" in t for t in report.watermark_types)
        assert report.characters_removed > 0
        assert "\u200b" not in report.cleaned_text

    def test_homoglyphs_cyrillic_in_latin(self):
        """Кириллические символы в латинском тексте."""
        # Заменяем 'a' на кириллическую 'а' (U+0430)
        text = "This is \u0430 test with homoglyphs"
        report = self.wd.detect(text)
        # Может быть обнаружено или нет в зависимости от контекста
        assert isinstance(report, object)

    def test_homoglyphs_latin_in_cyrillic(self):
        """Латинские символы в кириллическом тексте."""
        # Вставка латинской 'o' в кириллическое слово
        text = "Этo тест с гомоглифами"  # 'o' is Latin here
        report = self.wd.detect(text)
        assert isinstance(report.cleaned_text, str)

    def test_invisible_unicode(self):
        """Невидимые Unicode символы."""
        text = "Text with\u2060invisible\u2061characters"
        report = self.wd.detect(text)
        assert isinstance(report.cleaned_text, str)

    def test_multiple_spaces(self):
        """Множественные пробелы (spacing steganography)."""
        text = "Word  word   word  word   word  word   word  word   word  word"
        report = self.wd.detect(text)
        # Если > 5 multi-space sequences, обнаруживается
        if "spacing_steganography" in report.watermark_types:
            assert "  " not in report.cleaned_text

    def test_trailing_spaces(self):
        """Хвостовые пробелы на строках."""
        lines = ["Line one   ", "Line two  ", "Line three   ",
                 "Line four  ", "Line five   "]
        text = "\n".join(lines)
        report = self.wd.detect(text)
        if "trailing_space_steganography" in report.watermark_types:
            for line in report.cleaned_text.split('\n'):
                assert line == line.rstrip(' ')

    def test_statistical_watermarks_clean(self):
        """Статистические водяные знаки — чистый текст."""
        text = " ".join(["The quick brown fox jumps over the lazy dog."] * 10)
        report = self.wd.detect(text)
        # Не должно быть statistical_bias на обычном тексте
        assert isinstance(report, object)

    def test_confidence_calculation(self):
        """Уверенность вычисляется правильно."""
        text = "Clean\u200b text\u200b with\u200b marks"
        report = self.wd.detect(text)
        assert 0 <= report.confidence <= 1

    def test_detect_watermarks_func(self):
        """Функция-обёртка detect_watermarks работает."""
        report = detect_watermarks("Test\u200b text", lang="en")
        assert report.has_watermarks
        assert report.cleaned_text == "Test text"


class TestWatermarkDetectorRussian:
    """Тесты детектора для русского текста."""

    def setup_method(self):
        self.wd = WatermarkDetector(lang="ru")

    def test_clean_russian(self):
        """Чистый русский текст без гомоглифов."""
        # Текст гарантированно без латинских подмен
        report = self.wd.detect("Это чистый текст")
        # Проверяем что как минимум обработал
        assert isinstance(report.cleaned_text, str)

    def test_zero_width_russian(self):
        """Zero-width в русском тексте."""
        text = "Это\u200b текст\u200b с\u200b водяными знаками."
        report = self.wd.detect(text)
        assert report.has_watermarks
        assert "\u200b" not in report.cleaned_text



"""Тесты стилистического отпечатка (P3.2)."""

from __future__ import annotations

import pytest

from texthumanize.stylistic import StylisticAnalyzer, StylisticFingerprint

# ───────────────────────── Sample texts ─────────────────────

_SHORT_EN = (
    "The cat sat on the mat. It was a sunny day. "
    "Birds were singing in the trees, and children played in the park. "
    "Everything seemed peaceful and calm. "
    "However, dark clouds gathered on the horizon. "
    "Rain was coming soon. The old man sighed."
)

_FORMAL_EN = (
    "The implementation of advanced algorithms necessitates careful "
    "consideration of computational complexity. Furthermore, the "
    "integration of multiple subsystems requires robust interfaces. "
    "Consequently, a systematic approach to software architecture "
    "becomes paramount. It is essential to evaluate performance "
    "metrics comprehensively. The proposed methodology adheres to "
    "established best practices in the field. Moreover, empirical "
    "validation confirms the efficacy of this framework."
)

_CASUAL_EN = (
    "So I went to the store. Got some milk, bread, the usual stuff. "
    "Then I ran into my old friend Mike! We hadn't talked in ages. "
    "He told me about his new job — pretty cool actually. "
    "We decided to grab coffee. Best decision ever. "
    "Turns out he lives nearby now. Small world, right?"
)

_LONG_RU = (
    "Искусственный интеллект стремительно развивается. Нейронные сети "
    "обрабатывают огромные массивы данных. Машинное обучение позволяет "
    "решать сложные задачи автоматически. Однако важно понимать "
    "ограничения этих технологий. Они не заменяют человеческий разум. "
    "Этические вопросы требуют внимательного рассмотрения. Общество "
    "должно контролировать развитие ИИ. Безопасность остаётся "
    "приоритетом номер один."
)


# ──────────────────── StylisticAnalyzer tests ───────────────

class TestStylisticAnalyzer:
    """Tests for StylisticAnalyzer.extract()."""

    def test_extract_basic(self):
        sa = StylisticAnalyzer(lang="en")
        fp = sa.extract(_SHORT_EN)
        assert fp.sentence_length_mean > 0
        assert fp.sentence_length_median > 0
        assert fp.sentence_length_std >= 0
        assert fp.commas_per_k >= 0
        assert fp.avg_word_length > 0
        assert 0 <= fp.vocabulary_richness <= 1.0

    def test_extract_empty_text(self):
        sa = StylisticAnalyzer(lang="en")
        fp = sa.extract("")
        assert fp.sentence_length_mean == 0.0
        assert fp.avg_word_length == 0.0

    def test_extract_short_text(self):
        sa = StylisticAnalyzer(lang="en")
        fp = sa.extract("Short.")
        # < 100 chars => empty fingerprint
        assert fp.sentence_length_mean == 0.0

    def test_formal_vs_casual_style(self):
        sa = StylisticAnalyzer(lang="en")
        formal_fp = sa.extract(_FORMAL_EN)
        casual_fp = sa.extract(_CASUAL_EN)

        # Formal text has longer sentences
        assert formal_fp.sentence_length_mean > casual_fp.sentence_length_mean
        # Formal text has longer words on average
        assert formal_fp.avg_word_length > casual_fp.avg_word_length

    def test_russian_text(self):
        sa = StylisticAnalyzer(lang="ru")
        fp = sa.extract(_LONG_RU)
        assert fp.sentence_length_mean > 0
        assert fp.avg_paragraph_length > 0

    def test_punctuation_detection(self):
        sa = StylisticAnalyzer(lang="en")
        fp = sa.extract(_FORMAL_EN)
        # Formal text should have commas
        assert fp.commas_per_k > 0

    def test_question_detection(self):
        sa = StylisticAnalyzer(lang="en")
        fp = sa.extract(_CASUAL_EN)
        # Casual text has questions
        assert fp.question_ratio > 0

    def test_exclamation_detection(self):
        sa = StylisticAnalyzer(lang="en")
        fp = sa.extract(_CASUAL_EN)
        # Casual text has exclamations
        assert fp.exclamation_ratio > 0

    def test_connector_start(self):
        sa = StylisticAnalyzer(lang="en")
        fp = sa.extract(_FORMAL_EN)
        # Formal text starts sentences with connectors
        assert fp.connector_start_ratio > 0


# ──────────────────── StylisticFingerprint tests ────────────

class TestStylisticFingerprint:
    """Tests for StylisticFingerprint methods."""

    def test_similarity_identical(self):
        sa = StylisticAnalyzer(lang="en")
        fp1 = sa.extract(_SHORT_EN)
        fp2 = sa.extract(_SHORT_EN)
        sim = fp1.similarity(fp2)
        assert sim > 0.99  # identical text => near-perfect similarity

    def test_similarity_different_styles(self):
        sa = StylisticAnalyzer(lang="en")
        fp_formal = sa.extract(_FORMAL_EN)
        fp_casual = sa.extract(_CASUAL_EN)
        sim = fp_formal.similarity(fp_casual)
        # Different styles should be less similar
        assert sim < 0.95

    def test_similarity_range(self):
        fp1 = StylisticFingerprint(
            sentence_length_mean=15.0,
            sentence_length_std=5.0,
            commas_per_k=20.0,
        )
        fp2 = StylisticFingerprint(
            sentence_length_mean=15.0,
            sentence_length_std=5.0,
            commas_per_k=20.0,
        )
        assert fp1.similarity(fp2) > 0.99

    def test_to_vector(self):
        fp = StylisticFingerprint(
            sentence_length_mean=15.0,
            sentence_length_std=5.0,
        )
        vec = fp._to_vector()
        assert len(vec) == 15
        assert all(isinstance(v, float) for v in vec)

    def test_empty_fingerprint_similarity(self):
        fp1 = StylisticFingerprint()
        fp2 = StylisticFingerprint()
        # Both empty → zero vectors
        sim = fp1.similarity(fp2)
        assert sim == 0.0  # norm is 0


# ──────────────────── Pipeline integration tests ────────────

class TestStylisticPipelineIntegration:
    """Tests for target_style in pipeline."""

    def test_target_style_in_options(self):
        from texthumanize.utils import HumanizeOptions
        opts = HumanizeOptions(target_style=None)
        assert opts.target_style is None

    def test_target_style_with_fingerprint(self):
        from texthumanize.utils import HumanizeOptions
        fp = StylisticFingerprint(sentence_length_mean=12.0)
        opts = HumanizeOptions(target_style=fp)
        assert opts.target_style is fp

    def test_pipeline_with_target_style(self):
        from texthumanize import StylisticAnalyzer, humanize

        # Extract a target style from casual text
        sa = StylisticAnalyzer(lang="en")
        target = sa.extract(_CASUAL_EN)

        result = humanize(
            _FORMAL_EN,
            lang="en",
            profile="web",
            intensity=50,
            target_style=target,
        )

        assert result.text != ""
        # Metrics should include style info
        assert "style_similarity_before" in result.metrics_before

    def test_pipeline_without_target_style(self):
        from texthumanize import humanize

        result = humanize(
            "This is a simple test. It should work fine.",
            lang="en",
            profile="web",
            intensity=30,
        )
        # No style_similarity when no target
        assert "style_similarity_before" not in result.metrics_before


# ──────────────────── Cross-language tests ──────────────────

class TestStylisticMultilang:
    """Test stylistic fingerprint across languages."""

    @pytest.mark.parametrize("lang", ["en", "ru", "uk", "de", "fr", "es"])
    def test_extract_lang(self, lang):
        sa = StylisticAnalyzer(lang=lang)
        text = _LONG_RU if lang in ("ru", "uk") else _SHORT_EN
        fp = sa.extract(text)
        assert fp.sentence_length_mean > 0

    def test_different_languages_different_fingerprints(self):
        sa_en = StylisticAnalyzer(lang="en")
        sa_ru = StylisticAnalyzer(lang="ru")
        fp_en = sa_en.extract(_SHORT_EN)
        fp_ru = sa_ru.extract(_LONG_RU)
        # Different languages produce different fingerprints
        assert fp_en.sentence_length_mean != fp_ru.sentence_length_mean



"""Тесты деканцеляризации."""

from texthumanize.decancel import Debureaucratizer


class TestDebureaucratizer:
    """Тесты деканцеляризации."""

    def test_basic_ru_decancel(self):
        """Базовая деканцеляризация русского текста."""
        db = Debureaucratizer(lang="ru", profile="web", intensity=100, seed=42)
        text = "Данный текст осуществляет описание процесса."
        result = db.process(text)
        # "данный" должен замениться
        assert "данный" not in result.lower() or "этот" in result.lower()

    def test_basic_uk_decancel(self):
        """Базовая деканцеляризация украинского текста."""
        db = Debureaucratizer(lang="uk", profile="web", intensity=100, seed=42)
        text = "Даний текст здійснює опис процесу."
        result = db.process(text)
        assert isinstance(result, str)
        assert len(result) > 0

    def test_basic_en_decancel(self):
        """Базовая деканцеляризация английского текста."""
        db = Debureaucratizer(lang="en", profile="web", intensity=100, seed=42)
        text = "We utilize this methodology to facilitate the process."
        result = db.process(text)
        assert isinstance(result, str)

    def test_phrase_replacement(self):
        """Фразовые канцеляризмы заменяются."""
        db = Debureaucratizer(lang="ru", profile="web", intensity=100, seed=42)
        text = "Необходимо отметить, что этот процесс представляет собой пример."
        result = db.process(text)
        assert isinstance(result, str)

    def test_low_intensity(self):
        """Низкая интенсивность → мало изменений."""
        db = Debureaucratizer(lang="ru", profile="web", intensity=10, seed=42)
        text = "Данный текст осуществляет описание процесса."
        result = db.process(text)
        assert isinstance(result, str)

    def test_formal_profile_minimal(self):
        """Формальный профиль → минимум замен."""
        db = Debureaucratizer(lang="ru", profile="formal", intensity=60, seed=42)
        text = "Данный текст осуществляет описание процесса."
        result = db.process(text)
        assert isinstance(result, str)

    def test_changes_tracked(self):
        """Изменения отслеживаются."""
        db = Debureaucratizer(lang="ru", profile="chat", intensity=100, seed=42)
        text = "Данный текст осуществляет описание функционирования процесса."
        db.process(text)
        # При высокой интенсивности и chat профиле должны быть замены
        # (но мы не гарантируем количество из-за random)
        assert isinstance(db.changes, list)

    def test_empty_text(self):
        """Пустой текст обрабатывается."""
        db = Debureaucratizer(lang="ru", seed=42)
        result = db.process("")
        assert result == ""

    def test_no_bureaucratic_text(self):
        """Текст без канцеляризмов не меняется."""
        db = Debureaucratizer(lang="ru", profile="web", intensity=100, seed=42)
        text = "Простой понятный текст без сложных слов."
        result = db.process(text)
        # Может быть некоторое совпадение случайных слов, но в целом мало изменений
        assert isinstance(result, str)


class TestDebureaucratizStructure:
    """Тесты структуры обработки."""

    def test_case_preservation(self):
        """Регистр первой буквы сохраняется."""
        db = Debureaucratizer(lang="ru", profile="chat", intensity=100, seed=42)
        text = "Данный вопрос важен."
        result = db.process(text)
        # Первая буква должна остаться заглавной
        if result != text:
            assert result[0].isupper()
